Package: algaeClassify
Title: Tools to Query the 'Algaebase' Online Database, Standardize
        Phytoplankton Taxonomic Data, and Perform Functional Group
        Classifications
Version: 2.0.6
Date: 2026-05-10
Authors@R: c(person("Vijay","Patil",email=c("vij.patil@gmail.com"),role=c("aut","cre")),
		   person("Torsten","Seltmann",role="aut"),
		   person("Nico","Salmaso",role="aut"),
		   person("Orlane","Anneville",role="aut"),
		   person("Marc","Lajeunesse",role="aut"),
		   person("Dietmar","Straile",role="aut"))
Author: Vijay Patil [aut, cre],
  Torsten Seltmann [aut],
  Nico Salmaso [aut],
  Orlane Anneville [aut],
  Marc Lajeunesse [aut],
  Dietmar Straile [aut]
Maintainer: Vijay Patil <vij.patil@gmail.com>
Description: Functions that facilitate the use of accepted taxonomic nomenclature, collection of
	functional trait data, and assignment of functional group classifications to phytoplankton
	species. Possible classifications include Morpho-functional group (MFG; Salmaso et al. 2015 
	<doi:10.1111/fwb.12520>) and CSR (Reynolds 1988; Functional morphology and the 
	adaptive strategies of phytoplankton. In C.D. Sandgren (ed). Growth and reproductive 
	strategies of freshwater phytoplankton, 388-433. Cambridge University Press, New York). 
	Versions 2.0.0 and later includes new functions for querying the 
	'algaebase' online taxonomic database (www.algaebase.org), however these functions require
	a valid API key that must be acquired from the 'algaebase' administrators. 
	Note that none of the 'algaeClassify' authors are affiliated with 'algaebase' in any way. Taxonomic 
	names can also be checked against a variety of taxonomic databases using 
	the 'Global Names Resolver' service via its API (<https://resolver.globalnames.org/api>). In addition,
	currently accepted and outdated synonyms, and higher taxonomy, can be extracted for lists of
	species from the 'ITIS' database via its JSON web service API.
	The 'algaeClassify' package is a product of the GEISHA (Global Evaluation of the Impacts of 
	Storms on freshwater Habitat and Structure of phytoplankton Assemblages), funded by CESAB 
    (Centre for Synthesis and Analysis of Biodiversity) and the U.S. Geological Survey John Wesley Powell Center for
	Synthesis and Analysis, with data and other support provided by members of GLEON 
	(Global Lake Ecology Observation Network). 
	DISCLAIMER: This software has been approved for release by the 
	U.S. Geological Survey (USGS). Although the software has been subjected to rigorous review, 
	the USGS reserves the right to update the software as needed pursuant to further analysis and 
	review. No warranty, expressed or implied, is made by the USGS or the U.S. Government as to the 
	functionality of the software and related material nor shall the fact of release constitute 
	any such warranty. Furthermore, the software is released on condition that neither the USGS 
	nor the U.S. Government shall be held liable for any damages resulting from its authorized 
	or unauthorized use.
Depends: R (>= 4.4.0)
URL: https://doi.org/10.5066/F7S46Q3F
Imports: lubridate, stats, curl, jsonlite, methods
License: CC0
Encoding: UTF-8
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-11 00:22:47 UTC; vpatil
Repository: CRAN
Date/Publication: 2026-05-11 05:10:07 UTC

# Generated by roxygen2: do not edit by hand

export(accum)
export(algaebase_genus_search)
export(algaebase_output_parse)
export(algaebase_search_df)
export(algaebase_species_search)
export(bestmatch)
export(date_mat)
export(genus_search_itis)
export(genus_species_extract)
export(get_apikey)
export(get_apikey_fromfile)
export(gnr_df)
export(gnr_simple)
export(itis_search_df)
export(mean_naomit)
export(mfg_csr_convert)
export(mfg_csr_convert_df)
export(phyto_ts_aggregate)
export(sampeff)
export(set_algaebase_apikey_header)
export(species_search_itis)
export(species_to_mfg)
export(species_to_mfg_df)
export(traits_to_csr)
export(traits_to_csr_df)
export(traits_to_mfg)
export(traits_to_mfg_df)
importFrom(methods,is)

# Package: algaeClassify

## Title: Determine Phytoplankton Functional Groups Based on Functional Traits

-   release v2.0.6
    -   a newer version of this software package may be available.

## Authors

Vijay Patil (ORCID: 0000-0002-9357-194X) U.S. Geological Survey - Alaska
Science Center\
Torsten Seltmann\
Nico Salmaso\
Orlane Anneville\
Marc Lajeunesse\
Dietmar Straile

## Suggested Citation

Patil, V.P., Seltmann, T., Salmaso, N., Anneville, O., Lajeunesse, M.,
Straile, D., 2025. algaeClassify (ver 2.0.1, October 2023): U.S.
Geological Survey software release, <https://doi.org/10.5066/F7S46Q3F>

## Contact

Vijay Patil [vpatil\@usgs.gov](mailto:vpatil@usgs.gov){.email} - U.S.
Geological Survey - Alaska Science Center; 4210 University Drive;
Anchorage, Alaska 99508 USA; 907-786-7178
[ascweb\@usgs.gov](mailto:ascweb@usgs.gov){.email}

## Software Requirements

-   Requires R (version 4.4.2 or later)
    -   available for free download from the Comprehensive R Archive
        Network (CRAN) <https://cran.r-project.org>

## Distribution
-   The U.S. Geological Survey through this repository
    <https://doi.org/10.5066/F7S46Q3F>
-   Comprehensive R Archive Network (CRAN) 
    <https://cran.r-project.org/package=algaeClassify>

## Package Overview

The goal of algaeClassify is to facilitate the analysis of taxonomic and
functional trait data for phytoplankton.

Functions that facilitate the use of accepted taxonomic nomenclature,
collection of functional trait data, and assignment of functional group
classifications to phytoplankton species. Possible classifications
include Morpho-functional group (MFG; [Salmaso et al. 2015]
Salmaso, N., Naselli-Flores, L. and Padisak, J., 2015. Functional 
classifications and their application in phytoplankton ecology. 
Freshwater Biology, 60(4), pp.603-619.) and CSR, based on 
[Reynolds 1988] (Functional morphology and the adaptive strategies 
 of phytoplankton. In C.D. Sandgren (ed). Growth and reproductive 
 strategies of freshwater phytoplankton, 388-433. Cambridge University 
 Press, New York) and [Reynolds 2006] 
 <https://doi.org/10.1017/CBO9780511542145>.

Versions 2.0.0 and later includes new functions for querying the
algaebase online taxonomic database (www.algaebase.org), however these
functions require a valid API key that must be acquired from the
algaebase admin. Note that none of the algaeClassify authors are
affiliated with algaebase in any way. Taxonomic names can also be
checked against a variety of taxonomic databases using the geographic
name resolution service (GNRS) via wrapper functions for the R taxize
package, with convenient output format and unlikely names for
phytoplankton taxa removed. In addition, currently accepted and outdated
synonyms, and higher taxonomy, can be extracted for lists of species
from the ITIS database using wrapper functions for the R ritis package.
The algaeClassify package is a product of the GEISHA (Global Evaluation
of the Impacts of Storms on freshwater Habitat and Structure of
phytoplankton Assemblages), funded by CESAB (Centre for Synthesis and
Analysis of Biodiversity) and the USGS John Wesley Powell Center for
Synthesis and Analysis, with data and other support provided by members
of GLEON (Global Lake Ecology Observation Network).

## Installation

To use the new algaebase search functions, you MUST install v2.0.0 or later from
code.usgs.gov at <https://doi.org/10.5066/F7S46Q3F>. If you have git installed,
you can also install the package through R with:

``` 
#the following will install the package from the code.usgs.gov 
#repository:
require(devtools)
install_git("https://code.usgs.gov/asc/algaeClassify",ref="main") 
#update packages if necessary, or choose 'none' 
```

Next, load the package and ensure you have the correct version
installed.

```
library(algaeClassify)
```
```
citation("algaeClassify")
```

## April 5, 2023: New functions for querying algaeBase
(www.algaebase.org)!!!! Algaebase search function examples

```
#check out the package
help(package="algaeClassify")

#View the new algaebase search functions:
help("algaebase_species_search")
help("algaebase_genus_search")
help("algaebase_search_df")
```

## Using your API key

The Algaebase functions require an API key. You can obtain one from
[Algaebase]<https://www.algaebase.org/api>.

There are several options for using your api key. 1) assigning it to an
R object, and using it in function calls

```
apikey<- "asasdfasdfasdfasfd" #(not a real key)
algaebase_genus_search(genus="Anabaena",apikey=apikey)
```

2)  Saving it in a text file (e.g. "keyfile.txt"). You will need to give
    the filename and path using the api_file function argument.

```
algaebase_genus_search(genus="Anabaena",api_file="keyfile.txt")
```

3)  Finally, you can set your key as an environment variable. To do so,
    open or create a .Renviron text file in your home directory. One way
    to do this is by running the following line:

```
file.edit("~/.Renviron")
```

Add a line to the .Renviron file like the following, but use your actual
key after the = symbol:

ALGAEBASE_APIKEY=yourKeyHere

Finally, save and close the file, then restart R for changes to take
effect. Once the ALGAEBASE_APIKEY variable is defined, you do not need
to specify it in the algaebase search functions.

## Allowing international encodings and special characters

When reading in a phytplankton species list with read.csv, we recommend that 
you specify the character encoding. In the United States, using UTF-8 should 
correctly preserve umlautes, accents, and other special characters.

```
#example code. not a real file.
phyto.data<-read.csv('my_phytoplankton_species_list.csv',encoding="UTF-8")

#Can also set encoding for the R session.
#This can be specified to happen at startup.
options(encoding="UTF-8")
```

## Algaebase search function examples:

You can search for a single genus

```
algaebase_genus_search(genus="Anabaena")
```

Or a genus and species name

```
algaebase_species_search("Anabaena","flos-aquae")

#There are several arguments for these functions.
#you can control whether to include higher taxonomy in the output
algaebase_genus_search(genus="Navicula",higher=TRUE)

#You can also choose to include the full species name with author and date in 
#output
algaebase_genus_search(genus="Navicula",higher=TRUE,long=TRUE)

#The default only returns exact matches and the most recent entry in algaebase,
#but you can override that behavior
algaebase_species_search(genus="Nitzschia",species="acicularis",
newest.only=FALSE,exact.matches.only=FALSE,long=TRUE)
```

It is also possible to include infraspecific names in searches. To do so,
the species argument should be a character string containing the species name 
and infraspecific name, separated by a space.

```
algaebase_species_search(genus="Fragilaria",species="vaucheriae capitellata")
```
Algaebase_species_search will return an error for search terms that include
infraspecific labels such as var., subsp., etc.

```
algaebase_species_search(genus="Fragilaria",
                         species="vaucheriae var. capitellata")
```

If desired, you can view the raw output in JSON format

```
algaebase_genus_search(genus="Cyclotella",higher=TRUE,print.full.json=TRUE)
```

## Preprocessing data with genus_species_extract()

The genus_species_extract function can be used to process a list of binomial
names before submitting an algaebase search. This function accepts
a data.frame containing a column of character strings with binomial names,
and appends columns for genus and species. If infraspecific names are present,
The species column will contain both the species and infraspecific names, with
any infraspecific labels or label abbreviations removed. Genus_species_extract()
will also convert any input character encodings to UTF-8 to ensure consistent
search behavior when special characters are present.

```
test.df<-data.frame(phyto_name="Fragilaria vaucheriae var. capitellata")
test.df<-genus_species_extract(test.df,phyto.name='phyto_name')
#view the new dataframe with genus and species columns, formatted for 
#algaebase search.
head(test.df)
```

In all cases, the output will return the currently accepted name, as
well as the name that was supplied by the user. There are columns
indicating whether the input name is currently accepted and whether an
exact match was found.


## Submit a list of names to Algaebase. 

You can submit a data.frame of phytoplankton names to algaebase. 
The data frame must have columns named genus and species. We recommend using
the genus_species_extract() function to create properly formatted columns of 
genus and species names.

This will only return 1 result per name. If there are no exact matches
it will return NA by default. You can set exact.matches.only =FALSE for partial 
matching, but this may result in more than one match returned per submitted 
name. If there is no match for genus+species it will search for a genus-only 
match or you can specify genus.only searches for the entire dataset.

```
data(lakegeneva) #load small example dataset
head(lakegeneva) #view example dataset

lakegeneva<-genus_species_extract(lakegeneva,phyto.name="phyto_name")
lakegeneva<-lakegeneva[!duplicated(lakegeneva$phyto_name),]

lakegeneva.algaebase<-algaebase_search_df(lakegeneva,higher=TRUE,
genus.name="genus",species.name="species")

head(lakegeneva.algaebase)

##partial matching for an incorrectly-spelled name:
genus<-"Terpsinoe" # should be Terpsinoë
species<-"musica"

umlaut.test.df<-data.frame(genus,species)

algaebase_search_df(umlaut.test.df) #no match

algaebase_search_df(umlaut.test.df,exact.matches.only=FALSE) 
#no species match, but the function tries a genus-only search next, and 
finds the correct genus name as a partial match when exact.matches.only = FALSE.
```

## Other taxonomic search functions 

Version 2.0.0 includes functions for
searching the ITIS database and for using the Global Names Resolver
(GNR). ITIS search functions are based on the **ritis** package, while
GNR search uses the API described at <https://verifier.globalnames.org/>.

```
#ITIS
genus_search_itis(genus="Mougeotia",higher=TRUE)

species_search_itis(genspp="Anabaena flos-aquae")
#This does not return a match in ITIS, but Anabaena flosaquae

#GNR (Global Names Resolver)
#GNR/taxize use fuzzy/partial matching, and search multiple databases.
#If you do not find a hit in algaebase or ITIS, you can try searching for a 
#partial match via GNR

name<-"Anabaena flos-aquae"
gnr_simple(name=name,sourceid=3) #Search ITIS
gnr_simple(name=name,sourceid=NULL) #search for matches from any source

species_search_itis(genspp="Anabaena flosaquae") #check itis f0r gnr best match.
```
## Functional group classifications

AlgaeClassify can also be used to assist users in classifying species
into one of several morphological/functional classifications: Morpho-
functional groups (MFG) [Salmaso et al. 2015] or the C-S-R framework as 
described for freshwater systems [Reynolds 1988; Reynolds 2006]

It is possible to derive MFG and CSR classifications by supplying trait values.
Relevant traits are surface area, volume and maximum linear dimension for CSR,
or presence/absence of flagella, colonies, filamentous growth forms, and 
other traits for MFG, as described in Salmaso et al. 2015.

We hope to add additional functional classifications in future versions, such as
the Reynolds Functional Group classification system. Please send suggestions to
vpatil@usgs.gov!

```
traits_to_csr(sav=0.2,msv=10,traitrange=traitranges)

traits_to_mfg(flagella = 1,size = "large",colonial = 1,filament = 0,
              centric = NA,gelatinous = 0,aerotopes = 0,
              class = "Euglenophyceae",order = "Euglenales")
```

Users may be able to retrieve relevant trait data from published sources. For 
convenience, algaeClassify has included tables of MFG and CSR trait values
for common European species, extracted from [Rimet et al. 2018].
<doi:10.1051/limn/2018009>.

```
data("csrTraits")
data("mfgTraits")
head(csrTraits)
head(mfgTraits)
```

Alternatively, the user may be able to match species to a library of species:MFG
classifications derived via expert assessment as part of the
GEISHA John Wesley Powell Center fellowship.
<https://www.usgs.gov/centers/john-wesley-powell-center-for-analysis-and-synthesis/science/global-evaluation-impacts>

This library can be used to classify lists of species or single species names.
However, many morphofunctional traits are plastic and MFG classifications may
vary for a single species depending on location and environment. Use with
caution!
```
data("species_mfg_library")
head(species_mfg_library)

species_to_mfg('Scenedesmus','bijuga')

data(lakegeneva)
lakegeneva<-genus_species_extract(lakegeneva,phyto.name='phyto_name')
lakegeneva.mfg<-species_to_mfg_df(lakegeneva)

head(lakegeneva.mfg)
```

If desired, species_to_mfg_df() can be run with a user-supplied library of 
species:MFG matches based on their own research, using the mfgDbase argument.
```
help("specis_to_mfg_df")
```

Finally, the user can crosswalk MFG and CSR classifications, based on 
[Salmaso et al. 2015] and [Reynolds 1988].
```
data(mfg_csr_library)
mfg_csr_convert(mfg="11a-NakeChlor")
```
## Visualizing and manipulating long-term phytoplankton datasets

This is a basic example which shows you how to use algaeClassify to 1) identify 
anomalies in atime-series of phytoplankton species 2) calculate aggregate 
abundance at a higher taxonomic level (genus) 3) re-plot species accumulation 
curves to see if the taxonomic standardization and aggregation to higher
taxonomy have resolved the anomalies.

*phyto_ts_aggregate()* can be used to aggregate phytoplankton timeseries based 
on abundance and grouping variables (genus, MFG, station id, etc.). If no 
abundance var is given, will aggregate to presence/absence.

*date_mat()* is used to transform long-term data into a matrix of abundance or
presence/absence, organized by taxa/group and date (day, month, or year). This
simplifies the process of formatting data for ordinations or other plots of 
abundance change over time.

*accum()* compares observed change in total species richness over time 
to simulated curves based on random draws with replacement from a list of all
unique species in the dataset. step discontinuities in the observed curve &
departures from the simulated range may signal sudden ecological or 
methodological changes (including changes in taxonomic standards, personnel,
or equipment).


```

data(lakegeneva) #load the demonstration dataset

accum(lakegeneva,phyto_name='phyto_name',column='biovol_um3_ml',n=100,
      datename='date_dd_mm_yy',dateformat='%d-%m-%y')

#clean up binomial names and extract genus and species to new columns
lakegeneva<-genus_species_extract(lakegeneva,phyto.name='phyto_name')

#aggregate abundance data to genus level
lakegeneva.genus<-phyto_ts_aggregate(lakegeneva,SummaryType='abundance',
                                      AbundanceVar='biovol_um3_ml',
                                      GroupingVar1='genus')
```

```
#plot accumulation curve again, but at genus level
accum(lakegeneva.genus,phyto_name='genus',column='biovol_um3_ml',n=100,
      datename='date_dd_mm_yy',dateformat='%Y-%m-%d')

#classify taxa to CSR and visualize relative abundance of CSR groups by month
data(lakegeneva)

lakegeneva<-genus_species_extract(lakegeneva,phyto.name='phyto_name')
lakegeneva<-species_to_mfg_df(lakegeneva)
lakegeneva<-mfg_csr_convert_df(lakegeneva,mfg='MFG')
csrAbundance.by.month<-date_mat(lakegeneva,abundance.var='biovol_um3_ml',
                                taxa.name='CSR',time.agg='month')

#make a simple heatmap of mean daily csr group abundance by month
#In this example dataset, Stress-tolerant taxa dominate for a short time in 
#mid summer, followed by Ruderal-disturbance tolerant species that thrive during
#fall overturn. In winter, spring, and summer, small and fast-reproducing 
#(C)ompetitors are most abundant.
stats::heatmap(csrAbundance.by.month,Rowv=NA,Colv=NA,
xlab='CSR group',ylab='Month')
```

#' Get value of algaebase API key from Environment variable
#' Return an error if variable not set.
#'
#' @export get_apikey
#' @return api key as character string (invisibly)

get_apikey <- function() {
  val <- Sys.getenv("ALGAEBASE_APIKEY")
  if (identical(val, "")) {
    stop("`ALGAEBASE_APIKEY` env var has not been set. Add it to your ~/.Renviron file")
  }
  invisible(val)
}

#' Add CSR functional group classifications to a dataframe of phytoplankton species, based on surface to volume ratio and
#' maximum linear dimension ranges proposed by Reynolds et al. 1988;2006
#'
#' @param df name of dataframe
#' @param msv character string with name of column that contains maximum linear dimension * surface to volume ratio values
#' @param sav character string with name of column that contains surface to volume ratio values
#' @param msv.source character string with reference source for distinguishing criteria
#' @param traitrange data frame with trait criteria for c,s,r groups. The included table
#'     can be replaced with user-defined criteria if desired. Measurements are:
#'     Surface area/volume ratio (sav), maximum linear dimension (mld) and mld*sav (msv).
#'
#' @export traits_to_csr_df
#'
#' @return a character string with one of 5 return values: C,CR,S,SR, or R
#'
#' @examples
#'
#' csr.df<-data.frame(msv=10,sav=1)
#'
#' csr.df$CSR<-traits_to_csr_df(csr.df,'msv','sav')
#'
#'print(csr.df)

traits_to_csr_df=function(df,sav,msv,msv.source='Reynolds 2006',traitrange=algaeClassify::traitranges)
{
  csr=vector(mode='character',length=dim(df)[1])

  for(i in 1:dim(df)[1])
  {
    csr[i]=traits_to_csr(df[[sav]][i],df[[msv]][i],msv.source,traitrange)
  }

  return(csr)
}

#' Functional Trait Database derived from Rimet et al. 
#'
#' @format A data frame with columns:
#' \describe{
#'  \item{phyto_name}{binomial scientific name}
#'  \item{genus}{genus name}
#'  \item{species}{species name}
#'  \item{Mobility.apparatus}{1/0 indicates presence/absence of flagella or motility}
#'  \item{Size}{character values 'large' or 'small'; based on 35 micrometer max linear dimension}
#'  \item{Colonial}{1/0 indicates typical colonial growth form or not}
#'	\item{Filament}{1/0 indicates filamentous growth form or not}
#'	\item{Centric}{1/0 indicates diatoms with centric growth form}
#'	\item{Gelatinous}{1/0 indicates presence/absence of mucilage}
#'	\item{Aerotopes}{1/0 indicates presence/absence of aerotopes}
#'	\item{Class}{Taxonomic class}
#'  \item{Order}{Taxonomic order}
#'	\item{MFG.fromtraits}{MFG classification using traits_to_mfg function}
#' }
#'
#' @docType data
#'
#' @usage data(mfgTraits)
#'
#' @keywords datasets
#' 
"mfgTraits"

#' Trait-based MFG classifications for common Eurasion/North American phytoplankton species.
#' See accompanying manuscript for sources
#'
#' @format A data frame with columns:
#' \describe{
#'  \item{genus}{genus name}
#'  \item{species}{species name}
#'  \item{MFG}{corresponding MFG classification based on Salmaso et al. 2015}
#'  \item{source}{literature or online source for MFG classification}
#'  }
#'
#' @docType data
#'
#' @usage data(species_mfg_library)
#'
#' @keywords datasets
#'
#' @references {Algaebase} \url{https://www.algaebase.org}
#' @references {Phycokey} \url{https://www.cfb.unh.edu/phycokey/phycokey.htm}
#' @references {Western Diatoms of North America} \url{https://diatoms.org}
#' @references {CyanoDB 2} \url{http://www.cyanodb.cz/}
#' @references {Nordic Microalgae} \url{https://nordicmicroalgae.org}
#' @references {Phytopedia} \url{https://phytoplankton.eoas.ubc.ca/}
#' @references {Kapustin, D., Sterlyagova, I. and Patova, E., 2019. Morphology of Chrysastrella paradoxa stomatocysts from the Subpolar Urals (Russia) with comments on related morphotypes. Phytotaxa, 402(6), pp.295-300.}

"species_mfg_library"

#' fuzzy partial matching between a scientific name and a list of possible matches
#'
#' @param enteredName Character string with name to check
#' @param possibleNames Character vector of possible matches
#' @param maxErr maximum number of different bits allowed for a partial match
#' @param trunc TRUE/FALSE. if true and no match, retry with last three letters truncated
#'
#' @export bestmatch
#'
#' @return a character string with the best match, or 'multiplePartialMatches'
#'
#' @examples
#' possibleMatches=c('Viburnum edule','Viburnum acerifolia')
#' bestmatch(enteredName='Viburnum edulus',possibleNames=possibleMatches)

bestmatch=function(enteredName,possibleNames,maxErr=3,trunc=TRUE)
{
  for(i in 0:maxErr)
  {
    match=agrep(enteredName,possibleNames,max.distance=i,value=TRUE)
    
    if(length(match)==1) 
    {
      return(match)
    }
    
    if(length(match)>1)
    {
      if(i==0)
      {
        exact.match=(match==enteredName)
        if(sum(exact.match)==1)
        {
          return(enteredName)
        }
      }else
      {
        return('multiplePartialMatch')
        
      }
    }
  }
  ##strip last three letters and try again
  if(trunc==TRUE)
  {
    len=nchar(enteredName)
    truncName=substr(enteredName,1,len-3)
    trunc=FALSE
    bestmatch(truncName,possibleNames,trunc=trunc)
  }else{
    return(NA) #return NA if no exact or partial match found
  }
}

#' Returns a CSR classification based on Morphofunctional group (MFG).
#' Correspondence based on Salmaso et al. 2015 and Reynolds et al. 1988
#'
#' @param mfg Character string with MFG name, following Salmaso et al. 2015
#'
#' @export mfg_csr_convert
#' 
#' @return A character string with values 'C','S','R','CR','SC','SR', or NA
#' 
#' @examples
#' 
#' mfg_csr_convert("11a-NakeChlor")

#returns a CSR classification for a single MFG
mfg_csr_convert<-function(mfg)
{
  if(mfg %in% algaeClassify::mfg_csr_library$MFG==FALSE){csr=NA}
  else
  {
  	csr=algaeClassify::mfg_csr_library$CSR[algaeClassify::mfg_csr_library$MFG==mfg]
  	return(csr)
  }

}

#' Aggregate phytoplankton timeseries based on abundance. Up to 3 grouping variables can be given:
#' e.g. genus, species, stationid, depth range.
#' If no abundance var is given, will aggregate to presence/absence of grouping vars.
#'
#' @param phyto.data data.frame
#' @param AbundanceVar character string with field name containing abundance data
#'	Can be NA if data is only a species list and aggregated presence/absence is desired.
#' @param SummaryType 'abundance' for a matrix of aggregated abundance,'presence.absence'
#'	for 1 (present) and 0 (absent).
#' @param DateVar      character string: field name for date variable. character or POSIX data.
#' @param GroupingVar1 character string: field name for first grouping variable. defaults to spp.
#' @param GroupingVar2 character string: name of additional grouping var field
#' @param GroupingVar3 character string: name of additional grouping var field
#' @param remove.rare  TRUE/FALSE. If TRUE, removes all instances of GroupingVar1 that occur < 5%
#'                     of time periods.
#' @param fun          function used to aggregate abundance based on grouping variables
#' @param format       character string: format for DateVar POSIXct conversion
#'
#' @export phyto_ts_aggregate
#'
#' @return a data.frame with grouping vars, date_dd_mm_yy, and abundance or presence/absence
#'
#' @examples
#' data(lakegeneva)
#' lakegeneva<-genus_species_extract(lakegeneva,'phyto_name')
#' lg.genera=phyto_ts_aggregate(lakegeneva,SummaryType='presence.absence',
#'                              GroupingVar1='genus')
#' head(lg.genera)


phyto_ts_aggregate=function(phyto.data,DateVar='date_dd_mm_yy',
							SummaryType=c('abundance','presence.absence'),
							AbundanceVar='biovol_um3_ml',GroupingVar1='phyto_name',GroupingVar2=NA,
							GroupingVar3=NA,remove.rare=FALSE,fun=sum,format='%d-%m-%y')
{

  SummaryType=SummaryType[1]

  phyto.data$date_dd_mm_yy=as.POSIXct(phyto.data[[DateVar]],format=format)

  if(remove.rare)
  {
    rare.tab=table(phyto.data[[GroupingVar1]],phyto.data$date_dd_mm_yy)
    rare.tab=ifelse(rare.tab>0,1,0)
    num.occur=dim(rare.tab)[2]
    freq.occur=apply(rare.tab,1,sum)
    rare.spp=row.names(rare.tab)[freq.occur<(num.occur*.05)]
    phyto.data=phyto.data[phyto.data[[GroupingVar1]] %in% rare.spp==FALSE,]
  }

  groupingvars=c(GroupingVar1,GroupingVar2,GroupingVar3)
  groupingvars=stats::na.omit(groupingvars)

  #option to create a presence/absence matrix even if abundances are known
  if(SummaryType=='presence.absence' & !is.na(AbundanceVar))
  {
	phyto.data$presence=0
	phyto.data$presence[!is.na(phyto.data[[AbundanceVar]]) & phyto.data[[AbundanceVar]]>0]=1
	AbundanceVar='presence'
	fun=max
  }

  if(is.na(AbundanceVar))
  {
    phyto.data$presence=1
    AbundanceVar='presence'
    fun=max ##presence absence if no abundance var- this assumes every row is a detected species at that time
  }


  for(i in 1:length(groupingvars))
  {
    phyto.data[[groupingvars[i]]][is.na(phyto.data[[groupingvars[i]]])]=-9999
  }

  groupingvars=paste(groupingvars,collapse=' + ')

  agg.func=stats::formula(paste(AbundanceVar,"~ date_dd_mm_yy + ",groupingvars))

  ts.agg=stats::aggregate(agg.func,data=phyto.data,FUN=fun)

  return(ts.agg)



}

#' Assign phytoplankton species to CSR functional groups, based on surface to volume ratio and
#' maximum linear dimension ranges proposed by Reynolds et al. 1988;2006
#'
#' @param sav numeric estimate of cell or colony surface area /volume ratio
#' @param msv numeric product of surface area/volume ratio and maximum linear dimension
#' @param msv.source character string with reference source for distinguishing criteria
#' @param traitrange data frame with trait criteria for c,s,r groups. The included table
#'     can be replaced with user-defined criteria if desired. Measurements are:
#'     Surface area/volume ratio (sav), maximum linear dimension (mld) and mld*sav (msv).
#'
#' @export traits_to_csr
#'
#' @return a character string with one of 5 return values: C,CR,S,R, or SR.
#'         CR and SR groups reflect overlap between criteria for the 3 main groups.
#'
#' @examples
#'
#' traits_to_csr(sav=0.2,msv=10,msv.source='Reynolds 2006',traitrange=traitranges)
#'
#'
#' @seealso{<https://powellcenter.usgs.gov/geisha>} for project information


traits_to_csr=function(sav,msv,msv.source='Reynolds 2006',traitrange=algaeClassify::traitranges)
{

  csr=NA
  #must be based on measurements in micrometers (^2, ^3)
  sav.vals=unlist(traitrange[traitrange$Measurement=='sav',2:7])

  ##default is to use MSV criteria from Reynolds 2006
  ##but can also use MLD criteria from Reynolds 1988

  ##either option uses SAV criteria from Reynolds et al. 1988
  ##Because Reynolds et al. 2006 does not contain SAV criteria

  if(is.na(sav) | is.na(msv)){csr=NA;return(csr)}


  if(msv.source == 'Reynolds 1988')
  {
    mld.vals=unlist(traitrange[traitrange$Measurement=='mld',2:7])
    msv.vals=sav.vals*mld.vals
  }else{
    msv.vals=unlist(traitrange[traitrange$Measurement=='msv',2:7])
  }

  if(sav>=sav.vals[1] & sav < sav.vals[4] &
     msv >=msv.vals[1] & msv <msv.vals[4]){csr = 'C'}

  if(msv < msv.vals[3] & sav >= sav.vals[5]){csr='C'}

  if(msv < msv.vals[4] & sav >= sav.vals[6]){csr='C'}

  if(sav>=sav.vals[5] & msv >= msv.vals[4]){csr='R'}

  if(sav <sav.vals[5] &
     msv < msv.vals[5]){csr = 'S'}
  if(sav>=sav.vals[3] & sav < sav.vals[6] &
     msv>=msv.vals[3] & msv < msv.vals[6]){csr = 'R'}

  if(sav >=sav.vals[3] & sav < sav.vals[6] &
     msv >=msv.vals[3] & msv < msv.vals[4]){csr = 'CR'}

  if(msv >=msv.vals[5] & sav < sav.vals[3]) {csr='SR'}

  return(csr)

}

#' Visually assess change in sampling effort over time (author: Dietmar Straile)
#'
#' @param b_data Name of data.frame object
#' @param column column name or number for field containing abundance (biomass,biovol, etc.)
#'               can be NA for presence absence
#' @param save.pdf TRUE/FALSE Should the output plot be saved to a file? defaults to FALSE
#' @param lakename Character string for labeling output plot
#' @param datecolumn Character String or number specifying dataframe field with date information
#' @param dateformat Character string specifying POSIX data format
#'
#' @export sampeff
#'
#' @return a time-series plot of minimum relative abundance over time. This should change
#'         systematically with counting effort.
#'
#' @examples
#' data(lakegeneva)
#' #example dataset with 50 rows
#'
#' sampeff(lakegeneva,column=6) #column 6 contains biovolume

sampeff = function(b_data,column,save.pdf=FALSE,lakename='',datecolumn='date_dd_mm_yy',dateformat='%d-%m-%y') {
  abundance=b_data[[column]]
  b_data$abundance=abundance

  if(is.numeric(datecolumn)){datecolumn=names(b_data)[datecolumn]} #convert from number to name

  b_data[[datecolumn]]=as.POSIXct(b_data[[datecolumn]],format=dateformat)

  b_data = subset(b_data, abundance > 0)

  sbio = stats::aggregate(stats::formula(paste('abundance ~',datecolumn)) , data = b_data, sum)
  names(sbio)[2] = 'sabundance'
  sbio2 = merge(b_data, sbio, by = datecolumn, all = TRUE)
  sbio2$relabundance = sbio2$abundance /sbio2$sabundance
  minbio = stats::aggregate(stats::formula(paste('relabundance ~',datecolumn)), data = sbio2, min)

  if(save.pdf){grDevices::pdf(paste('sampeff',lakename,'.pdf',sep=''))}

  graphics::plot(I(100 * relabundance) ~ date_dd_mm_yy, data = minbio, log = 'y', xlab = '', ylab = 'Minimum contribution [%]')

  if(save.pdf){grDevices::dev.off()}
}

#' Wrapper function to apply species_phyto_convert() across a data.frame
#'
#' @param phyto.df Name of data.frame. Must have character fields named 'genus' and 'species'
#' @param flag Resolve ambiguous MFG: 1 = return(NA), 2 = manual selection
#' @param mfgDbase specify library of species to MFG associations.
#'
#' @export species_to_mfg_df
#'
#' @return input data.frame with a new character column of MFG classifications
#' and diagnostic information
#'
#'
#' @examples
#' data(lakegeneva)
#' #example dataset with 50 rows
#'
#' new.lakegeneva <- genus_species_extract(lakegeneva,'phyto_name')
#' new.lakegeneva <- species_to_mfg_df(new.lakegeneva)
#' head(new.lakegeneva)

species_to_mfg_df <- function(phyto.df,flag=1,mfgDbase=NA)
{
  phyto.len<-dim(phyto.df)[1]
  na.vec<-rep(NA,length=phyto.len)
  # new edit, specify category of columns before end. This appears to be working well for me. Is it faster than using rbind()?
  mfgs<-data.frame(MFG=factor(na.vec, levels = factor(unique(algaeClassify::species_mfg_library$MFG))),ambiguous.mfg=as.numeric(na.vec),genus.classification=as.numeric(na.vec),partial.match=as.numeric(na.vec),flag=as.numeric(na.vec))
  for(i in 1:phyto.len)
  {
    mfgs[i, ] <- species_to_mfg(phyto.df$genus[i],phyto.df$species[i],flag=flag,mfgDbase=mfgDbase)
  }

  # phyto.df<-phyto.df[,names(phyto.df) %in% c('MFG','ambiguous.mfg','genus.classification','partial.match','flag')==FALSE,]
  phyto.df$MFG=c(as.character(paste(mfgs[,1]))) # to make it work I had to include paste()
  # phyto.df$MFG[which(phyto.df$MFG == "NA")] <- NA # weird trick to make it back to the format you had... There is probably a better way to do that
  # phyto.df$MFG<-c(as.character(mfgs[,1]))
  phyto.df$ambiguous.mfg=c(as.character(mfgs[,2]))
  phyto.df$genus.classification=c(as.character(mfgs[,3]))
  phyto.df$partial.match=c(as.character(mfgs[,4]))
  phyto.df$flag=c(as.character(mfgs[,5]))
  return(phyto.df)
}

#' Wrapper function for several functions in ritis::
#' Searches ITIS database for matches to a genus name
#'
#' @param genus Character string. genus name to search for in ITIS
#' @param higher Boolean. If TRUE, add higher taxonomic classifications to output
#'
#' @export genus_search_itis
#'
#' @return input data.frame with matches, current accepted names, synonyms, and higher taxonomy
#'
#'
#' @examples
#' \donttest{
#' genus='Anabaena'
#' genus_search_itis(genus,higher=FALSE)
#' }

genus_search_itis<-function(genus,higher=FALSE)
{
  suppressWarnings(rm(list="res.df"))
  if(is.na(genus))
  {
    res.df=data.frame(matched.name=NA,match=0,
                      orig.name.accepted=0,
                      orig.name=genus,genus.only=1,synonyms="")
    if(higher){
      higher.df<-data.frame(Kingdom=NA,Subkingdom=NA,Infrakingdom=NA,
                            Phylum=NA,Class=NA,Subclass=NA,Order=NA,Family=NA)
      res.df<-cbind(res.df,higher.df)
    }
    return(res.df)
  }

  orig.name=genus
  genus<-strsplit(genus,split=' ')[[1]][1]

  raw<-tryCatch(
    itis_json_get("searchByScientificName", list(srchKey=genus), timeout=10),
    error=function(e){
      message("Warning: ITIS search returned an error: ",conditionMessage(e),
              "\nThe ITIS service may be temporarily unavailable.")
      NULL
    }
  )
  s <- if(!is.null(raw) && is.data.frame(raw$scientificNames)) raw$scientificNames else NULL

  if(is.null(s)||nrow(s)==0){
    res.df=data.frame(matched.name=NA,match=0,
                      orig.name.accepted=0,
                      orig.name=orig.name,genus.only=1,synonyms="")
    if(higher){
      higher.df<-data.frame(Kingdom=NA,Subkingdom=NA,Infrakingdom=NA,
                            Phylum=NA,Class=NA,Subclass=NA,Order=NA,Family=NA)
      res.df<-cbind(res.df,higher.df)
    }
    return(res.df)
  }

  # Filter: exact genus-only match (no species epithet), exclude Animalia
  genus.names <- s[s$kingdom != 'Animalia' & !is.na(s$unitName1) &
                   s$unitName1 == genus & (is.na(s$unitName2) | s$unitName2 == ''), ]
  genus.names <- genus.names[!is.na(genus.names$tsn), ]

  if(nrow(genus.names)>1){
    genus.names=genus.names[1,]
  }
  if(nrow(genus.names)==0){
    res.df=data.frame(matched.name=NA,match=0,
                      orig.name.accepted=0,
                      orig.name=genus,genus.only=1,synonyms="")
    if(higher){
      higher.df<-data.frame(Kingdom=NA,Subkingdom=NA,Infrakingdom=NA,
                            Phylum=NA,Class=NA,Subclass=NA,Order=NA,Family=NA)
      res.df<-cbind(res.df,higher.df)
    }
  }else{

    tsn=as.numeric(genus.names$tsn)

    raw_an<-tryCatch(
      itis_json_get("getAcceptedNamesFromTSN", list(tsn=tsn), timeout=10),
      error=function(e){
        message("Warning: could not retrieve accepted name from ITIS: ",conditionMessage(e))
        NULL
      }
    )
    accepted_names <- if(!is.null(raw_an) && is.data.frame(raw_an$acceptedNames))
      raw_an$acceptedNames else list()

    if(length(accepted_names)==0){
      orig.name.accepted=1
      accepted.name=genus
    }else{
      orig.name.accepted=0
      accepted.name=accepted_names$acceptedName
    }

    raw_syn<-tryCatch(
      itis_json_get("getSynonymNamesFromTSN", list(tsn=tsn), timeout=10),
      error=function(e){
        message("Warning: could not retrieve synonyms from ITIS: ",conditionMessage(e))
        NULL
      }
    )
    synonyms_df <- if(!is.null(raw_syn) && is.data.frame(raw_syn$synonyms))
      raw_syn$synonyms else NULL
    synonyms <- ifelse(is.null(synonyms_df)||nrow(synonyms_df)==0, "",
                       paste(synonyms_df$sciName, collapse=','))

    res.df<-data.frame(matched.name=accepted.name,
                       match=1,orig.name.accepted=orig.name.accepted,
                       orig.name=genus,genus.only=1,synonyms=synonyms)
    if(higher==TRUE)
    {
      ranks=c("Kingdom","Subkingdom","Infrakingdom","Phylum",
              "Class","Subclass","Order","Family")
      raw_h<-tryCatch(
        itis_json_get("getFullHierarchyFromTSN", list(tsn=tsn), timeout=10),
        error=function(e){
          message("Warning: could not retrieve higher taxonomy from ITIS: ",conditionMessage(e))
          NULL
        }
      )
      higher.tax <- if(!is.null(raw_h) && is.data.frame(raw_h$hierarchyList))
        raw_h$hierarchyList else NULL

      if(is.null(higher.tax)||nrow(higher.tax)==0){
        higher.df<-data.frame(Kingdom=NA,Subkingdom=NA,Infrakingdom=NA,
                              Phylum=NA,Class=NA,Subclass=NA,Order=NA,Family=NA)
      }else{
        # Direct API uses camelCase: rankName / taxonName
        higher.tax<-higher.tax[match(ranks, higher.tax$rankName) ,]
        higher.df<-data.frame(t(higher.tax$taxonName))
        names(higher.df)<-ranks
      }
      res.df<-cbind(res.df,higher.df)
    }
  }
  return(res.df)
}

#' example dataset from lake Geneva, Switzerland
#'
#' @format A data frame with columns:
#' \describe{
#'  \item{lake}{lake name}
#'  \item{phyto_name}{phytoplankton species name}
#'  \item{month}{month of sampling}
#'  \item{year}{year of sampling}
#'  \item{date_dd_mm_yy}{date of sampling}
#'  \item{biovol_um3_ml}{biovolume}
#' }
#'
#' @docType data
#'
#' @usage data(lakegeneva)
#'
#' @keywords datasets
#' 
"lakegeneva"

#' checks species names against a variety of online databases
#' supports fuzzy partial matching, using the Global Names Resolver
#' (https://resolver.globalnames.org/). Modified on 11/18/2025 by
#' Vijay Patil (vpatil@usgs.gov) for
#' algaeClassify v2.0.5 (pending approval on CRAN).
#'
#' @param name character string binomial scientific name to resolve
#' @param sourceid integer vector with data source ids.
#' see https://resolver.globalnames.org/sources/
#' @param best_match boolean. Should the best match be returned based on score?
#' @param fuzzy_uninomial boolean. Use fuzzy matching for uninomial names?
#' @param name_type Specify format of matched names. Options are
#' 'canonical_simple' (canonical binomial name), 'canonical_full' (with subspecies
#' or subgenera), or 'with_context' (with author and year appended).
#' @param higher boolean: Return higher taxonomic classifications?
#'
#' @export gnr_simple
#'
#' @return new data.frame with name matches, column indicating match type
#' and scores from Global Names Resolver (https://resolver.globalnames.org/).
#' Will contain a row of NAs if no matches found
#'
#' @examples
#' #Visit https://resolver.globalnames.org/data_sources to see all possible
#' #data sources for name checking.
#' name<-"Aphanazomenon flos-aquae"
#' #sourceid=3 for ITIS database
#' gnr_simple(name,sourceid=3) #search for best match from ITIS
#' gnr_simple(name,sourceid=NULL,best_match=FALSE) #search for all matches from any source

gnr_simple<-function(name,sourceid=NULL,best_match=TRUE,fuzzy_uninomial=TRUE,
                     name_type="canonical_full",higher=FALSE)
{
  #base API string
  gnrs.string<-"https://verifier.globalnames.org/api/v1/verifications/"

  gnrs.name<-gsub(' ','+',name)
  gnrs.string<-paste0(gnrs.string,gnrs.name,'?')

 if(!is.null(sourceid)){
    data_sources<-paste(sourceid,collapse='%7C')
    gnrs.string<-paste0(gnrs.string,"data_sources=",data_sources,"&")
  }

  if(best_match){
    gnrs.string<-paste0(gnrs.string,paste0("all_matches=false"),"&")
  }else{
    gnrs.string<-paste0(gnrs.string,paste0("all_matches=true"),"&")
  }

  if(fuzzy_uninomial){
    gnrs.string<-paste0(gnrs.string,"fuzzy_uninomial=true","&")
  }else{
    gnrs.string<-paste0(gnrs.string,"fuzzy_uninomial=false","&")
  }

  gnrs.string<-sub("&$","",gnrs.string)

  con<-curl::curl(gnrs.string)

  results<-try(readLines(con,warn=FALSE),silent=TRUE)

  if(is(results,"try-error"))
  {
    close(con)
    stop("No matches or too many requests. Wait several minutes and retry your search, or select a different data source.")
  }else
  {
    results.pretty<-jsonlite::prettify(results)
    close(con)
  }


  #transform to r list of lists
  result.list<-jsonlite::fromJSON(results.pretty)
  if(result.list$names$matchType=='NoMatch'){
   stop("No Matches")
  }

  #results dataframe
  if(is.null(result.list$names)){
    output<-data.frame(input_name=name,
                       match_name=NA,
                       match_score=NA,
                       match_type=NA,
					   taxonomic_status=NA,
                       data_source=NA,
                       data_import_date=NA
    )
    if(higher){
     output<-cbind(data.frame(Kingdom=NA,Phylum=NA,Class=NA,Order=NA,Family=NA),
                   output)
    }
    return(output)

  }

if(!best_match & 'results' %in% names(result.list$names)){
  results.output<-as.data.frame(result.list$names$results)
}else{
 results.output<- as.data.frame(result.list$names$bestResult)
}

match.name<-if(name_type=='canonical_simple'){
                  results.output$currentCanonicalSimple
               }else if(name_type=='with_context'){
                 results.output$currentName
               }else{
                 results.output$currentCanonicalFull
                 #currentCanonicalFull is default behavior
               }

  output<-data.frame(input_name=name,
                     match_name=match.name,
                    match_score=results.output$sortScore,
                    match_type=result.list$names$matchType,
                    taxonomic_status=results.output$taxonomicStatus,
                    data_source=results.output$dataSourceTitleShort,
                    data_import_date=results.output$entryDate
                    )
  if(higher){
    n<-nrow(results.output)
    higher.df<-data.frame(Kingdom=rep(NA,n),Phylum=rep(NA,n),Class=rep(NA,n),
                          Order=rep(NA,n),Family=rep(NA,n))
    higher.groups.keep<-tolower(names(higher.df))
    if(!is.null(results.output$classificationPath)){
      #not all sources have higher taxonomy. Pad with NAs if missing.
      higher.taxonomy<-results.output$classificationPath
      higher.taxonomy<-sapply(higher.taxonomy,strsplit,split='|',fixed=TRUE)
      higher.classes<-results.output$classificationRanks
      higher.classes<-sapply(higher.classes,strsplit,split='|',fixed=TRUE)
      for(i in 1:n){
        higher.df[i,]<-higher.taxonomy[[i]][match(higher.groups.keep,
                                                  tolower(higher.classes[[i]]))]
      }

    }
    output<-cbind(higher.df,output)
  }

  return(output)

}

#' Helper function for parsing output from algaebase
#' @param x list object containing output from an algaebase query
#' @param field.name character string
#'
#' @export algaebase_output_parse
#'
#' @return selected output variable as character vector
algaebase_output_parse<-function(x,field.name) {
  
  res<-x[[field.name]]
  if(is.null(res)){res=NA}
  return(res)
}

#' Returns a CSR classification based on Morphofunctional group (MFG).
#' Correspondence based on Salmaso et al. 2015 and Reynolds et al. 1988
#'
#' @param phyto.df dataframe containing a character field containing MFG classifications
#' @param mfg Character string with MFG name, following Salmaso et al. 2015
#'
#' @export mfg_csr_convert_df
#' 
#' @return A dataframe with an additional field named CSR, containing CSR classifications or NA
#' 
#' @examples
#' 
#' data(lakegeneva)
#' lakegeneva<-genus_species_extract(lakegeneva,'phyto_name')
#' lakegeneva<-species_to_mfg_df(lakegeneva)
#' lakegeneva<-mfg_csr_convert_df(lakegeneva,mfg='MFG')
#' head(lakegeneva)

mfg_csr_convert_df=function(phyto.df,mfg)
{
  csrs<-vector(length=dim(phyto.df)[1])
  for(i in 1:dim(phyto.df)[1])
  {
    csrs[i]=mfg_csr_convert(phyto.df[[mfg]][i])
  }
  phyto.df$CSR=csrs
  return(phyto.df)
}

#' Transform a phytoplankton timeseries into a matrix of abundances for ordination
#'
#' @param phyto.df Name of data.frame object
#' @param abundance.var Character string: field containing abundance data.
#'	Can be NA if the dataset only contains a species list for each sampling date.
#' @param summary.type 'abundance' for a matrix of aggregated abundance,'presence.absence'
#'	for 1 (present) and 0 (absent).
#' @param taxa.name Character string: field containing taxonomic identifiers.
#' @param date.name Character string: field containing date.
#' @param format Character string: POSIX format string for formatting date column.
#' @param time.agg Character string: time interval for aggregating abundance. default is day.
#' @param fun function for aggregation. default is mean, excluding NA's.
#'
#' @export date_mat
#'
#' @return A matrix of phytoplankton abundance, with taxa in rows and time in columns.
#'         If time.agg = 'monthyear', returns a 3dimensional matrix (taxa,month,year).
#'         If abundance.var = NA, matrix cells will be 1 for present, 0 for absent
#'
#' @examples
#' data(lakegeneva)
#' #example dataset with 50 rows
#'
#' geneva.mat1<-date_mat(lakegeneva,time.agg='month',summary.type='presence.absence')
#' geneva.mat2<-date_mat(lakegeneva,time.agg='month',summary.type='abundance')
#'
#' geneva.mat1
#' geneva.mat2

date_mat<-function(phyto.df,abundance.var='biovol_um3_ml',summary.type='abundance',taxa.name='phyto_name',
                   date.name='date_dd_mm_yy',format='%d-%m-%y',
                   time.agg=c('day','month','year','monthyear'),fun=mean_naomit)
{
  time.agg<-time.agg[1]

  if(summary.type=='presence.absence' & !is.na(abundance.var)) #option to create a presence/absence matrix even if abundances are known
  {
	phyto.df$presence=0
	phyto.df$presence[!is.na(phyto.df[[abundance.var]]) & phyto.df[[abundance.var]]>0]=1
	abundance.var='presence'
  }

  if(is.na(abundance.var)) #if only a species list, create a presence/absence matrix
  {
    phyto.df$presence=1
    abundance.var='presence'
	summary.type='presence.absence'
  }

  phyto.df[[date.name]]=as.character(phyto.df[[date.name]])
  phyto.df$date_dd_mm_yy=as.POSIXct(phyto.df$date_dd_mm_yy,format=format)

  phyto.df[[taxa.name]]=as.character(phyto.df[[taxa.name]])

  if(time.agg=='month')
  {
    phyto.df$month=lubridate::month(phyto.df$date_dd_mm_yy)
    phyto.agg=stats::aggregate(stats::formula(paste(abundance.var,'~',taxa.name,'+ month')),data=phyto.df,FUN=fun)

    phyto.mat=tapply(phyto.agg[[abundance.var]],list(phyto.agg$month,phyto.agg[[taxa.name]]),fun)
    phyto.mat[is.na(phyto.mat)]=0

    return(phyto.mat)
  }else if(time.agg=='year')
  {
    phyto.df$year=lubridate::year(phyto.df$date_dd_mm_yy)
    phyto.agg=stats::aggregate(stats::formula(paste(abundance.var,'~',taxa.name,'+ year')),data=phyto.df,FUN=fun)

    phyto.mat=tapply(phyto.agg[[abundance.var]],list(phyto.agg$year,phyto.agg[[taxa.name]]),fun)
    phyto.mat[is.na(phyto.mat)]=0

    return(phyto.mat)
  }else if(time.agg == 'yearmonth')
  {
    phyto.df$year=lubridate::year(phyto.df$date_dd_mm_yy)
    phyto.df$month=lubridate::month(phyto.df$date_dd_mm_yy)

    phyto.agg=stats::aggregate(stats::formula(paste(abundance.var,'~',taxa.name,'+ year +month')),data=phyto.df,FUN=mean.nona)

    phyto.mat=tapply(phyto.agg[[abundance.var]],list(phyto.agg$month,phyto.agg[[taxa.name]],phyto.agg$year),mean.nona)
    phyto.mat[is.na(phyto.mat)]=0

    return(phyto.mat)
  }

  mean.nona=function(x) mean(x[!is.na(x)])

  phyto.agg=stats::aggregate(stats::formula(paste(abundance.var,'~',taxa.name,'+',date.name)),data=phyto.df,FUN=mean.nona)

  phyto.mat=tapply(phyto.agg[[abundance.var]],list(phyto.agg[[date.name]],phyto.agg[[taxa.name]]),mean.nona)
  phyto.mat[is.na(phyto.mat)]=0

  return(phyto.mat)

}

#' Compute mean value while ignoring NA's
#'
#' @param x A numeric vector that may contain NA's
#'
#' @export mean_naomit
#'
#' @return the mean value
#'
#' @examples
#' data(lakegeneva)
#' #example dataset with 50 rows
#'
#' mean_naomit(lakegeneva$biovol_um3_ml)

mean_naomit<-function(x) {mean(x[!base::is.na(x)])}

#' Wrapper function to apply gnr_simple across a data.frame or list of species
#' names. Modified on 11/18/2025 by Vijay Patil (vpatil@usgs.gov) for
#' algaeClassify v2.0.5 (pending approval on CRAN).
#'
#' Provides convienent output with a row per name, to streamline merging with
#' original data.
#'
#' @param df data.frame containing names to check
#' @param name.column integer or character string with column name containing
#' 		  species names
#' @param sourceid integer vector with data source ids.
#' see https://resolver.globalnames.org/sources/
#' @param fuzzy_uninomial boolean. Use fuzzy matching for uninomial names?
#' @param name_type Specify format of matched names. Options are
#' 'canonical_simple' (canonical binomial name), 'canonical_full' (with subspecies
#' or subgenera), or 'with_context' (with author and year appended).
#' @param higher boolean: Return higher taxonomic classifications?
#'
#' @export gnr_df
#'
#' @return new data.frame original names (input_name), information about match
#' type,the best match (match_name), taxonomic status, and other output from
#' gnr_simple(). Will contain a row of NAs if no matches were found for a name.
#'
#' @examples
#' data(lakegeneva)
#' #example dataset with 50 rows
#'
#' lakegeneva<- genus_species_extract(lakegeneva,'phyto_name')
#' lakegeneva$genus_species <- trimws(paste(lakegeneva$genus,
#'	lakegeneva$species))
#'
#' #checking for matches from all GNRS sources, first 5 rows:
#' lakegeneva.namematches <- gnr_df(lakegeneva,"genus_species")
#' lakegeneva.namematches

gnr_df<-function(df,name.column,sourceid=NULL,
                 fuzzy_uninomial=TRUE, name_type="canonical_full",higher=FALSE)
{
  #base API string
  gnrs.string<-"https://verifier.globalnames.org/api/v1/verifications/"

  name<-df[,name.column]

  gnrs.name<-gsub(' ','+',name)
  if(length(gnrs.name)>1){
    gnrs.name<-paste(gnrs.name,collapse='%7C')
  }

  gnrs.string<-paste0(gnrs.string,gnrs.name,'?')

  #add search parameters
  if(!is.null(sourceid)){
    data_sources<-paste(sourceid,collapse='%7C')
    gnrs.string<-paste0(gnrs.string,"data_sources=",data_sources,"&")
  }

  #df search only returns best match to ensure 1 match per taxa.
  gnrs.string<-paste0(gnrs.string,paste0("all_matches=false"),"&")

  if(fuzzy_uninomial){
    gnrs.string<-paste0(gnrs.string,"fuzzy_uninomial=true","&")
  }else{
    gnrs.string<-paste0(gnrs.string,"fuzzy_uninomial=false","&")
  }

  gnrs.string<-paste0(gnrs.string,"species_group=true")

  con<-curl::curl(gnrs.string)

  results<-try(readLines(con,warn = FALSE),silent=TRUE)

  if(is(results,"try-error"))
  {
    close(con)
    stop("No matches or too many requests. Wait several minutes and retry your search, or select a different data source.")
  }else
  {
    results.pretty<-jsonlite::prettify(results)
    close(con)
  }

  #transform to r list of lists
  result.list<-jsonlite::fromJSON(results.pretty)

  #results dataframe
  if(is.null(result.list$names) |
     (length(unique(result.list$names$matchType))==1 && unique(result.list$names$matchType %in% 'NoMatch'))){
    output<-data.frame(input_name=name,
                       match_name=NA,
                       match_score=NA,
                       match_type=NA,
                       taxonomic_status=NA,
                       data_source=NA,
                       data_import_date=NA
    )
    if(higher){
      output<-cbind(data.frame(Kingdom=NA,Phylum=NA,Class=NA,Order=NA,Family=NA),
                    output)
    }
    return(output)

  }

  results.output<-result.list$names$bestResult

  match.name<-if(name_type=='canonical_simple'){
    results.output$currentCanonicalSimple
  }else if(name_type=='with_context'){
    results.output$currentName
  }else{
    results.output$currentCanonicalFull
    #currentCanonicalFull is default behavior
  }

  output<-data.frame(input_name=name,
                     match_name=match.name,
                     match_score=results.output$sortScore,
                     match_type=result.list$names$matchType,
                     taxonomic_status=results.output$taxonomicStatus,
                     data_source=results.output$dataSourceTitleShort,
                     data_import_date=results.output$entryDate)

  if(higher){
    n<-nrow(results.output)
    higher.df<-data.frame(Kingdom=rep(NA,n),Phylum=rep(NA,n),Class=rep(NA,n),
                          Order=rep(NA,n),Family=rep(NA,n))
    higher.groups.keep<-tolower(names(higher.df))
    if(!is.null(results.output$classificationPath)){
      #not all sources have higher taxonomy. Pad with NAs if missing.
      higher.taxonomy<-results.output$classificationPath
      higher.taxonomy<-sapply(higher.taxonomy,strsplit,split='|',fixed=TRUE)
      higher.classes<-results.output$classificationRanks
      higher.classes<-sapply(higher.classes,strsplit,split='|',fixed=TRUE)
      for(i in 1:n){
        higher.df[i,]<-higher.taxonomy[[i]][match(higher.groups.keep,
                                                  tolower(higher.classes[[i]]))]
      }

    }

    output<-cbind(higher.df,output)

  }

  return(output)
}

# Package-level environment holding a persistent curl handle.
# Reusing the same handle lets libcurl keep the TCP+TLS connection to
# www.itis.gov alive across sequential requests (HTTP keep-alive), avoiding
# a fresh handshake on every call.
.itis_pool <- new.env(parent = emptyenv())

# Internal helper: GET one ITIS JSON web service endpoint with an explicit
# timeout. Returns parsed list or NULL on any connectivity/parse error.
# Handles charset detection from Content-Type to cope with Latin-1 encoded
# author names in the ITIS web service responses.
itis_json_get <- function(endpoint, params = list(), timeout = 10) {
  base <- "https://www.itis.gov/ITISWebService/jsonservice/"
  if (length(params) > 0) {
    query <- paste(
      names(params),
      vapply(params, function(x) utils::URLencode(as.character(x), reserved = TRUE), ""),
      sep = "=", collapse = "&"
    )
    url <- paste0(base, endpoint, "?", query)
  } else {
    url <- paste0(base, endpoint)
  }

  # Reuse persistent handle so libcurl keeps the connection alive between calls
  if (!exists("handle", envir = .itis_pool, inherits = FALSE)) {
    .itis_pool$handle <- curl::new_handle()
  }
  h <- .itis_pool$handle
  curl::handle_setopt(h, timeout = timeout, followlocation = 1L)

  res <- tryCatch(curl::curl_fetch_memory(url, handle = h), error = function(e) NULL)
  if (is.null(res) || res$status_code != 200L) return(NULL)

  # Detect charset from Content-Type response header (same logic as ritis/crul)
  headers_text <- rawToChar(res$headers)
  ct_lines     <- strsplit(headers_text, "\r\n")[[1]]
  ct_line      <- ct_lines[grepl("content-type", ct_lines, ignore.case = TRUE)]
  enc <- if (length(ct_line) > 0) {
    m <- regmatches(ct_line[1],
                    regexpr("(?i)charset=([^;[:space:]]+)", ct_line[1], perl = TRUE))
    if (length(m) > 0) sub("(?i)charset=", "", m, perl = TRUE) else "UTF-8"
  } else "UTF-8"

  text <- tryCatch(
    iconv(rawToChar(res$content), from = enc, to = "UTF-8", sub = "byte"),
    error = function(e) NULL
  )
  if (is.null(text)) return(NULL)
  tryCatch(jsonlite::fromJSON(text), error = function(e) NULL)
}

#' MFG-CSR correspondence based on CSR-trait relationships in Reynolds et al. 1988
#' and MFG-trait relationships in Salmaso et al. 2015
#'
#' @format A data frame with columns:
#' \describe{
#'  \item{MFG}{full MFG name from Salmaso et al. 2015}
#'  \item{CSR}{CSR classification including intermediate classes}
#' }
#'
#' @docType data
#'
#' @usage data(mfg_csr_library)
#'
#' @keywords datasets
#'
"mfg_csr_library"

#' Assign morphofunctional groups to a dataframe of functional traits and higher taxonomy
#'
#' @param dframe An R dataframe containing functional trait information and higher taxonomy
#' @param arg.names Character string of column names corresponding to arguments for traits_to_mfg()
#' @return A character vector containing morpho-functional group (MFG) designations
#'
#' @export traits_to_mfg_df
#'
#' @examples
#' #create a two-row example dataframe of functional traits
#' func.dframe=data.frame(flagella=1,size=c("large","small"),colonial=0,filament=0,centric=NA,
#'                        gelatinous=0,aerotopes=0,class="Euglenophyceae",order="Euglenales",
#'                        stringsAsFactors=FALSE)
#'
#' #check the dataframe
#' print(func.dframe)
#'
#' #run the function to produce a two-element character vector
#' func.dframe$MFG<-traits_to_mfg_df(func.dframe,c("flagella","size","colonial",
#'                                  "filament","centric","gelatinous",
#'                                  "aerotopes","class","order"))
#'
#' print(func.dframe)
traits_to_mfg_df<-function(dframe,arg.names=c("flagella",
                                              "size",
                                              "colonial",
                                              "filament",
                                              "centric",
                                              "gelatinous",
                                              "aerotopes",
                                              "class",
                                              "order"))
{
  #dframe is a data frame of functional traits
  #arg.names is a vector of column names that match the arguments for traits_to_mfg()
  mfg.from.traits=""
  for(i in 1:dim(dframe)[1])
  {
    mfg.from.traits[i]=traits_to_mfg(dframe[[arg.names[1]]][i],
                                     dframe[[arg.names[2]]][i],
                                     dframe[[arg.names[3]]][i],
                                     dframe[[arg.names[4]]][i],
                                     dframe[[arg.names[5]]][i],
                                     dframe[[arg.names[6]]][i],
                                     dframe[[arg.names[7]]][i],
                                     dframe[[arg.names[8]]][i],
                                     dframe[[arg.names[9]]][i])
  }

  return(mfg.from.traits)
}

#'Split a dataframe column with binomial name into genus and species columns.
#'Plots change in species richness over time, generates species accumulation curve, and
#'compares SAC against simulated idealized curve assuming all unique taxa have equal probability
#'of being sampled at any point in the time series. (author Dietmar Straile)
#'
#' @param b_data Name of data.frame object
#' @param phyto_name Character string: field containing phytoplankton id (species, genus, etc.)
#' @param column column name or number for field containing abundance (biomass,biovol, etc.).
#'               Can be NA if the dataset only contains a species list for each sampling date.
#' @param n number of simulations for randomized ideal species accumulation curve
#' @param save.pdf TRUE/FALSE- should plots be displayed or saved to a pdf?
#' @param lakename optional character string for adding lake name to pdf output
#' @param datename character string name of b_data field containing date
#' @param dateformat character string: posix format for datename column
#'
#' @export accum
#'
#' @return a two panel plot with trends in richness on top, and cumulative richness vs. simulated
#'         accumulation curve on bottom
#'
#' @examples
#' data(lakegeneva)
#' #example dataset with 50 rows
#' head(lakegeneva)
#'
#' accum(b_data=lakegeneva,column='biovol_um3_ml',n=10,save.pdf=FALSE)

accum = function(b_data, phyto_name='phyto_name',column=NA, n=100, save.pdf=FALSE,lakename='',
                 datename='date_dd_mm_yy',dateformat='%d-%m-%y') {

  oldpar <- graphics::par(no.readonly = TRUE) 
  on.exit(graphics::par(oldpar))#reset to original par() settings if function exits.
  
	b_data[[datename]]=as.character(b_data[[datename]])

  b_data$date_dd_mm_yy = as.POSIXct(b_data[[datename]],format=dateformat)
  graphics::par(mfcol=c(2,1))
  graphics::par(mar=c(3,4,1,1))
  if(is.na(column))
  {
    b_data$presence=1
    column=grep('presence',names(b_data))
  }
  b_data = subset(b_data, b_data[, column] > 0)

  ##assigning phytoplankton id column to phyto_name
  b_data[[phyto_name]]=as.character(b_data[[phyto_name]])

  ntaxa0 = as.data.frame(table(b_data$date_dd_mm_yy, b_data[[phyto_name]]))

  ntaxa0 = subset(ntaxa0, ntaxa0$Freq > 0)

  ntaxa = as.data.frame(table(ntaxa0$Var1))
  ntaxa$date_dd_mm_yy = as.POSIXct(as.character(ntaxa$Var1),format='%Y-%m-%d')

  if(save.pdf)
  {
    grDevices::pdf(paste('phytoSAC',lakename,'.pdf',sep=''))
    graphics::par(mfcol=c(2,1))
    graphics::par(mar=c(3,4,1,1))
  }
  graphics::plot(ntaxa$Freq ~ ntaxa$date_dd_mm_yy, pch = 19, ylab = 'number of taxa', xlab = '')

  sbio = stats::aggregate(b_data[, column] ~ b_data[[phyto_name]] + b_data[[datename]],
                         FUN=sum)

  abundance.var=ifelse(is.numeric(column),names(b_data)[column],column)
  names(sbio)=c('phyto_name','date_dd_mm_yy',abundance.var)

  sbio$date_dd_mm_yy=as.POSIXct(sbio$date_dd_mm_yy,format='%Y-%m-%d')

  rich = as.data.frame(table(sbio$date_dd_mm_yy),stringsAsFactors = FALSE)
  rich$Var1=as.POSIXct(rich$Var1,format='%Y-%m-%d')

  rich=rich[match(unique(sbio$date_dd_mm_yy),rich$Var1),]
  first = stats::aggregate(sbio$date_dd_mm_yy ~ sbio$phyto_name, data = sbio, min)
  names(first)=c('phyto_name','date_dd_mm_yy')

  first$n = 0


  num.taxa=length(rich$Var1)
  freq=rich$Freq

  row.end=cumsum(rep(sum(freq),n))

  row.start=1
  for(i in 2:n)
  {
    row.start[i]=row.end[i-1]+1
  }

  simul.length=n * sum(freq)


  nruns = data.frame (phyto_name=vector(length=simul.length),date_dd_mm_yy = vector(length=simul.length),n=vector(length=simul.length))

  for (j in 1:n){
    tt = data.frame( phyto_name = NULL,date_dd_mm_yy = NULL)
    for (i in 1:length(rich$Var1)) { #for each date
      x1 = subset(sbio, sbio$date_dd_mm_yy == unique(sbio$date_dd_mm_yy)[i])
      #take aggregate data from that date

      ttt =  data.frame(phyto_name = sample(first$phyto_name, as.numeric(table(x1$date_dd_mm_yy)), replace = FALSE), date_dd_mm_yy = unique(sbio$date_dd_mm_yy)[i],stringsAsFactors = FALSE)
      #sample randomly from list of all taxa seen, draw number of unique taxa seen that day

      tt = rbind(tt, ttt)
      #repeat for each date
    }
    tt$n=j
    nruns[row.start[j]:row.end[j],] = tt
  }
  nruns$date_dd_mm_yy = as.POSIXct(nruns$date_dd_mm_yy, origin = '1970-01-01')
  nruns$n=rep(1:n,each=sum(freq))
  mue = rbind(first, nruns)

  mue = mue[order(mue$date_dd_mm_yy),]

  graphics::plot(c(1:dim(subset(mue, n == 0))[1]) ~ date_dd_mm_yy, data = subset(mue, n == 0), ylab  = 'cum number of taxa', xlab = '')
  mue = subset(mue, mue$date_dd_mm_yy > 0)

  for(i in 1:n){
    xx = subset(mue, mue$n == i)
    xx$date_dd_mm_yy=as.POSIXct(xx$date_dd_mm_yy,origin = '1970-01-01')

    xxx = stats::aggregate(date_dd_mm_yy ~ phyto_name, data = xx, min)
    xxx = xxx[order(xxx$date_dd_mm_yy),]

    graphics::lines(c(1:dim(xxx)[1]) ~ date_dd_mm_yy, data = xxx, type = 'l', col = 'grey')
  }

  if(save.pdf)
  {
    grDevices::dev.off()
  }

  graphics::par(oldpar)#reset to original par() settings.

}

#' Search algaebase for information about a list of phytoplankton names
#'
#' @param df data frame containing columns with genus and species names
#' @param apikey valid key for algaebase API as character string
#' @param genus.only boolean: should searches be based solely on the genus name?
#' @param genus.name name of data.frame column that contains genus names
#' @param species.name name of data.frame column that contains species names
#' @param handle curl handle with API key. Will be created if not present.
#' @param higher boolean should higher taxonomy be included in output?
#' @param print.full.json boolean returns raw json output if TRUE. Default is FALSE (return R data frame)
#' @param long boolean return long output including full species name and authorship, and entry date from algaebase.
#' @param exact.matches.only boolean should results be limited to exact matches?
#' @param api_file path to text file containing a valid API key
#' @param sleep.time delay between algaebase queries (in seconds). Should be at least 1 second if querying more than 10 names at once.
#'
#' @export algaebase_search_df
#'
#' @importFrom methods is
#'
#' @return data frame that may include: accepted.name (currently accepted synonym if different from input name), input.name (name supplied by user), input.match (1 if exact match, else 0), currently.accepted (1=TRUE/0=FALSE), genus.only (1=genus search/0=genus+species search),higher taxonomy (kingdom,phylum,class,order,family), genus, species (always NA for genus search), infraspecies name (always NA for genus search), long.name (includes author and date if given), taxonomic.status (currently accepted, synonym, or unverified), taxon.rank (taxonomic rank of accepted name (genus, species, infraspecies), mod.date (date when entry was last modified in algaebase).
#'
#' @examples
#' \dontrun{
#' data(lakegeneva)
#' #example dataset with 50 rows
#'
#' new.lakegeneva <- genus_species_extract(lakegeneva,'phyto_name')
#' lakegeneva.algaebase<-algaebase_search_df(new.lakegeneva[1:10,],higher=TRUE,long=TRUE)
#' head(lakegeneva.algaebase)}
#'
algaebase_search_df<-function(df,apikey=NULL,handle=NULL,genus.only=FALSE,
                              genus.name='genus',species.name='species',
                              higher=TRUE,print.full.json=FALSE,
                              long=FALSE,exact.matches.only=TRUE,
                              api_file=NULL,sleep.time=1)
{

  algaebase_df<-data.frame()

  nrows=nrow(df)
  err.row.func<-function(df,index=0,genus.only=FALSE,
                         genus.name='genus',
                         species.name='species',higher=TRUE,long=TRUE){

      if(genus.only){input.name<-df[[genus.name]][index]}else{
      input.name<-trimws(paste(df[[genus.name]][index],df[[species.name]][index]))
    }

    err.df.row<-data.frame(kingdom=NA,phylum=NA,class=NA,order=NA,family=NA,
                           genus=NA,species=NA,infrasp=NA,taxonomic.status=NA,
                           currently.accepted=NA,accepted.name=NA,genus.only=NA,
                           input.name=input.name,input.match=0,taxon.rank=NA,
                           mod.date=NA,long.name=NA,authorship=NA)


    if(higher==FALSE){
      err.df.row<-err.df.row[,names(err.df.row)%in% c('kingdom','phylum','class','order','family')==FALSE]
    }

    if(long==FALSE){
      err.df.row<-err.df.row[,names(err.df.row)%in% c('long.name','authorship','taxonomic.status','mod.date')==FALSE]
    }

    return(err.df.row)
  }

    for(i in 1:nrows){


      Sys.sleep(sleep.time)
      genus=df[[genus.name]][i]
      species=df[[species.name]][i]

      #if no species match, try genus
      #if no genus match, fill row with NAs
      #if no exact matches specified, will return newest hit
      #not smart enough yet to return closest non-exact match.
      #don't get stuck trying to find all genera ever if no genus name supplied.
      if(genus==''|is.na(genus)|is.null(genus)){
                tmp<-err.row.func(df=df,index=i,
                        genus.only=genus.only,
                        genus.name=genus.name,
                        species.name=species.name,
                        higher=higher,long=long)}else if(genus.only|species==''|is.na(species)|is.null(species)){

        tmp<-try(algaebase_genus_search(genus=genus,apikey=apikey,handle=handle,
                                    higher=higher,
                                    print.full.json=FALSE,
                                    newest.only=TRUE,long=long,
                                    exact.matches.only=exact.matches.only,
                                    api_file=api_file,
                                    return.higher.only=FALSE),silent=TRUE)

      }else{
        tmp<-try(algaebase_species_search(genus=genus,species=species,apikey=apikey,handle=handle,
                                    higher=higher,
                                    print.full.json=FALSE,
                                    newest.only=TRUE,long=long,
                                    exact.matches.only=exact.matches.only,
                                    api_file=api_file),silent=TRUE)
        if(is(tmp,'try-error')){
          tmp<-try(algaebase_genus_search(genus=genus,apikey=apikey,handle=handle,
                                          higher=higher,
                                          print.full.json=FALSE,
                                          newest.only=TRUE,long=long,
                                          exact.matches.only=exact.matches.only,
                                          api_file=api_file,
                                          return.higher.only=FALSE),silent=TRUE)
        }
      }

      if(is(tmp,'try-error')){
        tmp<-err.row.func(df=df,index=i,
                                  genus.only=genus.only,
                                  genus.name=genus.name,
                                  species.name=species.name,
                                  higher=higher,long=long)
      }

      print(paste0(round(100*i/nrows),"% complete"))
      algaebase_df<-rbind(algaebase_df,tmp)
    }


    return(algaebase_df)
}

#only difference is return.higher.only

#' Search algaebase for information about a genus of phytoplankton
#'
#' @param genus genus name as character string
#' @param apikey valid key for algaebase API as character string
#' @param handle curl handle with API key. Will be created if not present.
#' @param higher boolean should higher taxonomy be included in output?
#' @param print.full.json boolean returns raw json output if TRUE. Default is FALSE (return R data frame)
#' @param newest.only boolean should results be limited to the most recent matching entry in algaebase?
#' @param long boolean return long output including full species name and authorship, and entry date from algaebase.
#' @param exact.matches.only boolean should results be limited to exact matches?
#' @param return.higher.only boolean should output only included higher taxonomy?
#' @param api_file path to text file containing a valid API key
#'
#' @export algaebase_genus_search
#'
#' @importFrom methods is
#'
#' @return data frame that may include: accepted.name (currently accepted synonym if different from input name), input.name (name supplied by user), input.match (1 if exact match, else 0), currently.accepted (1=TRUE/0=FALSE), genus.only (1=genus search/0=genus+species search),higher taxonomy (kingdom,phylum,class,order,family), genus, species (always NA for genus search), infraspecies name (always NA for genus search), long.name (includes author and date if given), taxonomic.status (currently accepted, synonym, or unverified), taxon.rank (taxonomic rank of accepted name (genus, species, infraspecies), mod.date (date when entry was last modified in algaebase).
#'
#' @examples
#' \dontrun{algaebase_genus_search("Anabaena")} #not run.
#'

algaebase_genus_search<-function(genus=NULL,apikey=NULL,handle=NULL,
                                 higher=TRUE,print.full.json=FALSE,
                                 newest.only=TRUE,long=FALSE,
                                 exact.matches.only=TRUE,
                                 return.higher.only=FALSE,
								 api_file=NULL){

  if(genus==''|is.na(genus)|is.null(genus))
  {
    stop("No genus name supplied")
  }

  genus.search.string<-paste0('https://api.algaebase.org/v1.3/genus?genus=',genus)

  #check for api key in Sys.env
  #read from file if not specified or available as env variable.
  if(is.null(api_file) & is.null(apikey)){
	apikey<-get_apikey()
  }else if(!is.null(api_file)){
	apikey<-get_apikey_fromfile(api_file)
  }

  #set curl handle if not supplied as an argument
  if(is.null(handle)){
    handle<-set_algaebase_apikey_header(apikey)
  }

  con <- curl::curl(genus.search.string, handle = handle)
  results<-jsonlite::prettify(try(readLines(con),silent=TRUE))
  if(is(results,"try-error"))
  {
    close(con)
    stop("No matches") #throw error for now.
    #will need to modify in wrapper so that it fills with NA instead.
  }

  close(con)

  if(print.full.json){ #can just return the raw output if that is what user wants
    print(results)
    return(results)

  }

  #transform to r list of lists
  result.list<-jsonlite::fromJSON(results)


  #objects
  pagination<-result.list[[1]]
  results.output<-result.list[[2]]
  # num.results<-pagination$`_total_number_of_results`
  # print(num.results) #remove this later- how many results are there?
  # names(results.output) #field names for each hit

  #all of this could be condensed in later versions of the function
  #but if it runs quickly then who cares.
  #can still add a delay between api requests if necessary?

  #adding extra option to be extract higher taxonomy for species searches
  if(return.higher.only){
    higher=TRUE; exact.matches.only=TRUE; newest.only=TRUE

  }

  if(higher==TRUE){
    kingdom<-algaebase_output_parse(results.output,"dwc:kingdom")
    phylum<-algaebase_output_parse(results.output,"dwc:phylum")
    taxonomic.class<-algaebase_output_parse(results.output,"dwc:class")
    taxonomic.order<-algaebase_output_parse(results.output,"dwc:order")
    taxonomic.family<-algaebase_output_parse(results.output,"dwc:family")
    taxonomic.genus<-algaebase_output_parse(results.output,"dwc:genus")

    higher.taxonomy<-data.frame(kingdom=kingdom,phylum,class=taxonomic.class,
                                  order=taxonomic.order,
                                  family=taxonomic.family,
                                genus=taxonomic.genus)

    mod.date<-algaebase_output_parse(results.output,"dcterms:modified")
    mod.date<-lubridate::ymd(mod.date) #set as date type so you can sort output based on latest modification

    if(return.higher.only==TRUE){
      if(exact.matches.only){
        if(sum(taxonomic.genus==genus)==0){stop("No exact matches found")
        }else{higher.taxonomy<-higher.taxonomy[taxonomic.genus==genus,]}
      }

      if(newest.only){
        higher.taxonomy<-higher.taxonomy[mod.date==max(mod.date),] #only retain the most recent edit
      }else{
        higher.taxonomy<-higher.taxonomy[order(mod.date,decreasing=TRUE),]
      }

    return(higher.taxonomy)
    }
  }

  long.name<-algaebase_output_parse(results.output,"dwc:scientificName")
  taxonomic.status<-algaebase_output_parse(results.output,"dwc:taxonomicStatus")
  taxonRank<-algaebase_output_parse(results.output,"dwc:taxonRank")
  authorship<-algaebase_output_parse(results.output,"dwc:scientificNameAuthorship")
  mod.date<-algaebase_output_parse(results.output,"dcterms:modified")
  mod.date<-lubridate::ymd(mod.date) #set as date type so you can sort output based on latest modification
  accepted.name<-algaebase_output_parse(results.output,"dwc:acceptedNameUsage")
  input.name=genus
  input.match=ifelse(genus==taxonomic.genus,1,0)
  currently.accepted=ifelse(taxonomic.status=='currently accepted taxonomically',1,0)
  accepted.name<-ifelse(currently.accepted==1,taxonomic.genus,accepted.name)

  output<-data.frame(genus=taxonomic.genus,species=NA,infrasp=NA,taxonomic.status,currently.accepted,accepted.name,genus.only=1,input.name=genus,
                     input.match,taxon.rank=taxonRank,mod.date,long.name,authorship)
  if(higher){output<-cbind(higher.taxonomy,output);
              output<-subset(output,select= c('accepted.name','input.name','input.match','currently.accepted','genus.only','kingdom','phylum','class','order','family','genus','species','infrasp',
                                                         'long.name','taxonomic.status','taxon.rank','mod.date','authorship'))}else{
              output<-subset(output,select=c('accepted.name','input.name','input.match','currently.accepted','genus.only','genus','species','infrasp',
                                                                        'long.name','taxonomic.status','taxon.rank','mod.date','authorship') )
                                                         }


  #only retain exact matches if asked.
  if(exact.matches.only){
    if(sum(output$input.match)==0){stop("No exact matches found")
    }else{output<-output[output$input.match==1,]}
  }

  if(newest.only){
    output<-output[output$mod.date==max(output$mod.date),] #only retain the most recent edit
  }else{
    output<-output[order(output$mod.date,decreasing=TRUE),]
  }

  if(!long){output<-output[,names(output) %in% c('long.name','authorship','taxonomic.status','mod.date')==FALSE]}






    return(output)
}

#' Assign MFG based on binary functional traits and taxonomy (Class and Order)
#'
#' @param flagella 1 if flagella are present, 0 if they are absent.
#' @param size Character string: 'large' or 'small'. Classification criteria is left to the user.
#' @param colonial 1 if typically colonial growth form, 0 if typically unicellular.
#' @param filament 1 if dominant growth form is filamentous, 0 if not.
#' @param centric 1 if diatom with centric growth form, 0 if not. NA for  non-diatoms.
#' @param gelatinous 1 mucilagenous sheath is typically present, 0 if not.
#' @param aerotopes 1 if aerotopes allowing buoyancy regulation are typically present, 0 if not.
#' @param class Character string: The taxonomic class of the species
#' @param order Character string: The taxonomic order of the species
#'
#' @export traits_to_mfg
#'
#' @return A character string of the species' morphofunctional group
#'
#' @examples
#' traits_to_mfg(flagella = 1,size = "large",colonial = 1,filament = 0,centric = NA,gelatinous = 0,
#'                aerotopes = 0,class = "Euglenophyceae",order = "Euglenales")


traits_to_mfg <- function(flagella = NA,
                         size = NA,
                         colonial = NA,
                         filament = NA,
                         centric = NA,
                         gelatinous = NA,
                         aerotopes = NA,
                         class = NA,
                         order = NA)
{
  mfg = NA
  if(flagella %in% 1 &
      class %in% c("Bacillariophyceae",
					"Coscinodiscophyceae",
					"Mediophyceae",
					"Fragilariophyceae")==FALSE) {
    #making sure that diatoms are excluded from this branch
    if (order %in% c("Volvocales",
					  "Chlamydomonadales")){
      if (colonial %in% 1) {
        mfg = "3b-ColoPhyto"
        }
      else {
          mfg = "3a-UnicPhyto"
      }
    }else if(class %in% 'Cryptophyceae'){
    #ensures that all motile cryptophytes go to 2d.
        mfg =  "2d-Crypto"
    }else
      if (size %in% "large") {
        if (class %in% c("Chrysophyceae",
					   "Haptophyceae",
					   "Synurophyceae",
					   "Phaeothamniophyceae")) {
          mfg = "1a-LargeChry"
          }
        else
          if (class %in% "Dinophyceae") {
          mfg = "1b-LargeDino"
          }
          else if (class %in% 'Euglenophyceae'){
            mfg = "1c-LargeEugl"
          }
        }
      else if(size %in% "small"){
        if (class %in% c("Chrysophyceae",
						 "Haptophyceae",
						 "Synurophyceae",
						 "Phaeothamniophyceae")) {
			mfg = "2a-SmallChry1"
          }
        else
          if (class %in% "Dinophyceae"){
            mfg = "2b-SmallDino"
          }
		else
		if (class %in% "Euglenophyceae") {
              mfg = "2c-SmallEugl"
          }
		}
    else {
          mfg = NA
      }
    }
  # first node break: flagella == 0
  else {
    if (class %in% c("Cyanophyceae",
					 "Cyanobacteria")){
          if (order %in% c("Nostocales","Hormogonales")){
            mfg = "5e-Nostocales"
          }
		else
      if (colonial %in% 1){
        if (order %in% "Oscillatoriales"){
          mfg = "5a-FilaCyano"
        }
        else
          if (size %in% "large") {
            if (aerotopes %in% 1){
              mfg = "5b-LargeVacC"
            }
            else {
              mfg = "5c-OtherChroo"
            }
          }
        else if (size %in% "small"){
          mfg = "5d-SmallChroo"
        }
      }
      else {
        mfg = "4-UnicCyano"
      }
    }
  else
    if (class %in% c("Bacillariophyceae",
				   "Coscinodiscophyceae",
				   "Mediophyceae",
				   "Fragilariophyceae")){
      if (size %in% "large") {
        if (centric %in% 1) {
          if (colonial %in% 1) {
            mfg = "6a1-LColCent"
          }
          else {
            mfg = "6a2-LUniCent"
          }
        }
        else {
          if (colonial %in% 1){
            mfg = "6b1-LColPenn"
          }
        else {
          mfg = "6b2-LUniPenn"
        }
	   }
      }
      else{
        if (centric %in% 1) {
          mfg = "7a-SmallCent"
        }
      else {
        mfg = "7b-SmallPenn"
      }
	 }
    }
  else
    if (colonial %in% 1){
      if (filament %in% 1) {
        if (class %in% c("Chlorophyceae",
						 "Ulvophyceae",
						 "Trebouxiophyceae")){
          mfg = "10a-FilaChlorp"
          }
        else
          if (class %in% c("Conjugatophyceae",
						   "Zygnematophyceae")){
          mfg = "10b-FilaConj"
          }
        else
          if (class %in% c("Xanthophyceae",
						   "Eustigmatophyceae")){
          mfg = "10c-FilaXant"
          }
      }
      else
        if (order %in% c("Chlorococcales",
						"Chlamydomonadales",
						"Tetrasporales")) {
            if (gelatinous %in% 1) {
            mfg = "11b-GelaChlor"
            }
            else {
            mfg = "11a-NakeChlor"
            }
          }
      else {
        mfg = "11c-OtherCol"
      }
        }
      else
    if (size %in% "large") {
      if (class %in% c("Chlorophyceae",
					   "Conjugatophyceae",
					   "Zygnematophyceae")) {
        mfg = "8a-LargeCoCh"
      }
      else {
        mfg = "8b-LargeUnic"
      }
    }
  else {
    if (class %in% c("Conjugatophyceae",
					 "Zygnematophyceae")){
      mfg = "9a-SmallConj"
    }
  else
    if (order %in% c("Chlorococcales","Sphaeropleales")) {
      mfg = "9b-SmallChlor"
    }
  else
    if (class %in% c("Chrysophyceae",
					 "Synurophyceae",
					 "Phaeothamniophyceae")) {
      mfg = "9c-SmallChry2"
    }
  else
  if(!is.na(class) & !is.na(order))
  {
    mfg = "9d-SmallUnic"
  }
  }
 }
  return(mfg)
}

#' Database of functional traits for MFG classification, derived from Rimet et al. 2019 
#'
#' @format A data frame with columns:
#' \describe{
#'  \item{phyto_name}{binomial scientific name}
#'  \item{genus}{genus name}
#'  \item{species}{species name}
#'  \item{SAV}{surface area:volume ratio}
#'  \item{MLD}{maximum linear dimension (micrometers)}
#'  \item{MSV}{product of SAV and MLD; unitless}
#'	\item{volume.um3}{cell or colony biovolume}
#'	\item{surface.area.um2}{biological unit (cell or colony) surface area accounting for mucilage}
#'	\item{Colonial}{1/0 indicates colonial growth form}
#'	\item{Number.of.cells.per.colony}{literature-based average colony abundance}
#'	\item{Geometrical.shape.of.the.colony}{Shape descriptions. See Rimet et al. 2019 for abbreviations}
#'  \item{traitCSR}{CSR classification using traits_to_CSR function and criteria from Reynolds 2006}
#' }
#'
#' @docType data
#'
#' @usage data(mfgTraits)
#'
#' @keywords datasets
#' 
"csrTraits"

#' surface/volume ratio and max linear dimension criteria for CSR
#' From Reynolds 1988 and Reynolds 2006
#'
#' @format A data frame with columns:
#' \describe{
#'  \item{Measurement}{measurement type}
#'  \item{C.min}{minimum value for C}
#'  \item{S.min}{minimum value for S}
#'  \item{R.min}{minimum value for R}
#'  \item{C.max}{maximum value for C}
#'  \item{S.max}{maximum value for S}
#'  \item{R.max}{maximum value for R}
#'  \item{units}{units of measurement}
#'  \item{source}{source for criteria}
#' }
#'
#' @docType data
#'
#' @usage data(traitranges)
#'
#' @keywords datasets
#'
"traitranges"

#' Conversion of a single genus and species name to a single MFG. Uses species.mfg.library
#'
#' @param genus Character string: genus name
#' @param species Character string: species name
#' @param flag Resolve ambiguous mfg: 1 = return(NA),2= manual selection
#' @param mfgDbase data.frame of species MFG classifications. Defaults to the supplied species.mfg.library data object
#'
#' @export species_to_mfg
#'
#' @return a data frame with MFG classification and diagnostic information.
#' ambiguous.mfg=1 if multiple possible mfg matches
#' genus.classification=1 if no exact match was found with genus + species name
#' partial.match=1 if mfg was based on fuzzy matching of taxonomic name.
#'
#' @examples
#' species_to_mfg('Scenedesmus','bijuga')
#' #returns "11a-NakeChlor"

species_to_mfg<-function(genus,species="",flag=1,mfgDbase=NA)#set flag to two if you want to
													 #manually resolve ambiguous mfg class.
  #default behavior is to set ambiguous classes to NA (flag=1)
{
  if(!is.data.frame(mfgDbase))
  {
	mfgDbase<-algaeClassify::species_mfg_library
  }

  mfgDbase<-mfgDbase[!duplicated(mfgDbase),]

  #create vector for indicator if name is ambiguous or not.
  ambiguous.mfg=0
  #vector indicating if mfg classification was based on genus-level match.
  genus.classification=0

  #vector indicating whether the bestmatch function was used or not.
  partial.match=0

  genus=gsub('Unknown ','',genus)
  if(species %in% mfgDbase$species==FALSE){species=''}#replacing spp., sp. etc. with ''

  #hardcoding a match
  if(genus=='Hyloraphidium'){genus='Hyaloraphidium'}

  if(species==''){genus.classification=1}
  #check for genus and species match first.
  mfg=mfgDbase$MFG[mfgDbase$genus==genus &
							  mfgDbase$species==species]
  #go to genus match
 if(length(unique(mfg)==1))
 {
   mfg=unique(mfg)
 }else{
   species=''
   mfg=mfgDbase$MFG[mfgDbase$genus==genus & mfgDbase$species==species]
   genus.classification=1
 }



  #if there is no genus only match, see if there is another species with the same genus
  if(length(unique(mfg))==0)
  {
    mfg=mfgDbase$MFG[mfgDbase$genus==genus]
    genus.classification=1
  }

  if(length(unique(mfg))==0)
  {
    #try for fuzzy genus matching
    genus.bestmatch<-bestmatch(enteredName=genus,possibleNames=unique(mfgDbase$genus))
    if(genus.bestmatch != 'multiplePartialMatches' & !is.na(genus.bestmatch))
    {
      mfg=mfgDbase$MFG[mfgDbase$genus==genus.bestmatch]
      genus.classification=1
      partial.match=1
    }
  }

  #now, mfg length should be 0, 1, or 2
  if(length(unique(mfg[!is.na(mfg)]))==2)#flag 2 means you can interactively
							#choose among two possible mfgs for a particular genus or species
  {
    ambiguous.mfg=1
    if(flag==1)
    {
      mfg=NA
    }else if (flag==2)
    {
      mfg=unique(mfg)
      cat(paste('\n two possible mfgs for the species: ',genus,species))
      cat()
      cat(paste('\n1:',mfg[1]))
      cat(paste('\n2:',mfg[2]))
      cat(paste('\n3:', NA))
      choice=as.numeric(readline(prompt='\nenter your choice: (1,2,3): \n'))
      mfg=mfg[choice]
    }
  }

  mfg<-unique(mfg[!is.na(mfg)])
  if(length(mfg)==0 )
  {
    mfg=NA
  }else
  {
    mfg=mfg[1]
  }

  mfg.df<-data.frame(MFG=mfg,
                     ambiguous.mfg=ambiguous.mfg,
                     genus.classification=genus.classification,
                     partial.match=partial.match,
                     flag=flag)
  return(mfg.df)
}

#' Add algaebase API key to curl handle
#' @param apikey character string with valid key 
#'
#' @export set_algaebase_apikey_header
#'
#' @return curl handle object
set_algaebase_apikey_header<-function(apikey=NULL)
{
  if(is.null(apikey)){
    stop("You must have a valid API key for www.algaebase.org")
  }
  h <- curl::new_handle(verbose = TRUE)
  curl::handle_setheaders(h,
                    "Content-Type" = "application/json",
                    "abapikey" = apikey)
  return(h)
}

#' Split a dataframe column with binomial name into genus and species columns.
#'
#' @param phyto.df Name of data.frame object
#' @param phyto.name Character string: field in phyto.df containing species name.
#'
#' @export genus_species_extract
#'
#' @return A data.frame with new character fields 'genus' and 'species'
#'
#' @examples
#' data(lakegeneva)
#' #example dataset with 50 rows
#'
#' head(lakegeneva) #need to split the phyto_name column
#' new.lakegeneva=genus_species_extract(lakegeneva,'phyto_name')
#'
#' head(new.lakegeneva)

genus_species_extract<-function(phyto.df,phyto.name)
#phyto.name is character string indicating column containing phytoplankton binomial names
{
  spp.list<-as.character(phyto.df[[phyto.name]])

  orig.spp.list=spp.list

  spp.list=iconv(spp.list, to='UTF-8') #switched from ASCII- will retain umlauts, accents, etc.
  spp.list=gsub('Cfr. ','',spp.list,ignore.case=TRUE)
  spp.list=gsub('cf ','',spp.list,ignore.case=TRUE)
  spp.list=gsub('cf.','',spp.list,ignore.case=TRUE)

  ##removing descriptors- may contain useful information, but are not a proper name
  spp.list=gsub('colony','',spp.list,ignore.case=TRUE)
    spp.list=gsub('colonies','',spp.list,ignore.case=TRUE)
	  spp.list=gsub('cells','',spp.list,ignore.case=TRUE)
	    spp.list[spp.list=='cell']=''

  ###cleaning up genus-only records
  genus.only.flag=rep(0,length(spp.list)) #flag for species names with spp. or sp. in them
  genus.only.flag[grep(' sp | sp. | spp | spp. | sp.$| spp.$| sp$| sp1| spp$',
					   spp.list,ignore.case=TRUE)]=1
					   #doesn't account for sp or spp at end of string

  ###flagging and cleaning up records with subspecies or vars
  ###good god these are all the different abbreviations for variety subspecies that I found!
  ###have to account for abbreviations with and without capitalization and upper/lower case
  var.flag=rep(0,length(spp.list)) #vector to indicate if a species name includes a var/subsp/etc.
  var.flag[grep(' var. | subsp. | ssp. | v. | morph. | gr. | mor. | var | subsp | ssp | v | morph |
				  gr | mor | aff. | aff | f | f. | tab. ',spp.list,ignore.case=TRUE)]=1

  spp.list=gsub(' var. | subsp. | ssp. | v. | morph. | gr. | mor. | var | subsp | ssp | v | morph |
				  gr | mor | aff. | aff | f | f. | tab. ',' ',spp.list,ignore.case=TRUE)

  ###trimming leftover trailing and leading whitespace
  spp.list=trimws(spp.list,'both')

  genus=sapply(spp.list,function(x) strsplit(x,split=' ')[[1]][1])
  species=sapply(spp.list,function(x) strsplit(x,split=' ')[[1]][2])
  species[is.na(species)]=''
  species=ifelse(species %in% c('sp.','spp.','sp','spp'),'',species)
  species[grep('[0-9]',species)]='' ##cut out species names with numbers in them.

  #for the search url, species and var are pasted together with a plus sign.
  #will have to be stripped out if you are returning the search species name.

  var=sapply(spp.list,function(x) strsplit(x,split=' ')[[1]][3])
  var<-trimws(var,'both')

  var.flag.test=!(grepl("^[[:upper:]]",var) | substr(var,1,1)=="(" | grepl("^[0-9]",var))
  var.flag[var.flag.test==1]=1

  #not using line below
  #var[grep('comb. nov.',orig.spp.list)]='comb.+nov.'

  #pasting species and subspecies/var into a single string.
  species[var.flag==1 & !is.na(var)]=paste(species[var.flag==1 & !is.na(var)],
		  var[var.flag==1 & !is.na(var)],sep=' ')

  genus[is.na(genus)]=''
  species[is.na(species)]=''
  species[genus.only.flag==1]=''

  genus=trimws(genus,'both')
  species=trimws(species,'both')

  phyto.df$genus=c(genus)
  phyto.df$species=c(species)

  return(phyto.df)
}

#' Retrieve taxonomic information from the algaebase online database (www.algaebase.org) based on a user-specified genus and species name . This function requires a valid API key for algaebase.
#'
#' @param genus genus name as character string
#' @param species species name as character string
#' @param apikey valid key for algaebase API as character string
#' @param handle curl handle with API key. Will be created if not present.
#' @param higher boolean should higher taxonomy be included in output?
#' @param print.full.json boolean returns raw json output if TRUE. Default is FALSE (return R data frame)
#' @param newest.only boolean should results be limited to the most recent matching entry in algaebase?
#' @param long boolean return long output including full species name and authorship, and entry date from algaebase.
#' @param exact.matches.only boolean should results be limited to exact matches?
#' @param api_file path to text file containing a valid API key
#'
#' @export algaebase_species_search
#'
#' @importFrom methods is
#'
#' @return data frame that may include: accepted.name (currently accepted synonym if different from input name), input.name (name supplied by user), input.match (1 if exact match, else 0), currently.accepted (1=TRUE/0=FALSE), genus.only (1=genus search/0=genus+species search),higher taxonomy (kingdom,phylum,class,order,family), genus, species (always NA for genus search), infraspecies name (always NA for genus search), long.name (includes author and date if given), taxonomic.status (currently accepted, synonym, or unverified), taxon.rank (taxonomic rank of accepted name (genus, species, infraspecies), mod.date (date when entry was last modified in algaebase).
#'
#' @examples
#'
#' \dontrun{algaebase_species_search("Anabaena flos-aquae")} #not run
#'
#'
algaebase_species_search<-function(genus,species,apikey=NULL,handle=NULL,
                                 higher=TRUE,print.full.json=FALSE,
                                 newest.only=TRUE,long=FALSE,exact.matches.only=TRUE,
								 api_file=NULL){

  #first, throw error if there is no genus name
  #this prevents the code from trying to search every possible name in the
  #database.
  if(genus==''|is.na(genus)|is.null(genus))
  {
    stop("No genus name supplied")
  }

  #parse infraspecific names from specificEpithet, if present
  #and create appropriate search query string.
  #genus_species_extract strips out infraspecific category names
  #so we will just look for the infraspecific name within the full scientific name.
  if(length(grep(" ",species)>0)){
    species.split=strsplit(species,split=' ')[[1]]
    sp=species.split[1]

    #remove weird descriptors from species
    sp<-gsub(r"{\s*\([^*].*}","",sp)
    sp<-gsub(">","",sp)
    sp<-gsub("<","",sp)

    infrasp=species.split[2]

    species.search.string<-paste0("https://api.algaebase.org/v1.3/species?genus=",
                                  genus,"&dwc:specificEpithet=",sp,
                                  "&dwc:scientificName=",infrasp)

    }else{
     species.search.string<-paste0("https://api.algaebase.org/v1.3/species?genus=",genus,
                                  "&dwc:specificEpithet=",species)
}



  #check for api key in Sys.env
  #read from file if not specified or available as env variable.
  if(is.null(api_file) & is.null(apikey)){
	apikey<-get_apikey()
  }else if(!is.null(api_file)){
	apikey<-get_apikey_fromfile(api_file)
  }

  #set curl handle if not supplied as an argument
  if(is.null(handle)){
    handle<-set_algaebase_apikey_header(apikey)
  }

  #submit query
  con <- curl::curl(species.search.string, handle = handle)

  #parse query results
  results<-try(readLines(con),silent=TRUE)
  if(is(results,"try-error"))
  {
    close(con)
    stop("No matches") #throw error for now.
    #will need to modify in wrapper so that it fills with NA instead.
  }

  results<-jsonlite::prettify(results)
  close(con) #need to close the connection asap.

  if(print.full.json){ #can just return the raw output if that is what user wants
    print(results)
    return(results)

  }
  #transform to r list of lists
  result.list<-jsonlite::fromJSON(results)

  #objects

  #so, if no match in the first page of output, you have to keep going.
  #need to deal with the issue of multiple pages more explicitly at some point
  #but it shouldn't be a huge problem most of the time.
  #for now you could return a flag or warning if there are multiple pages of results.

  pagination<-result.list[[1]]
  results.output<-result.list[[2]]


  # num.results<-pagination$"_total_number_of_results"
  # num.pages<-pagination$"_total_number_of_pages"

  #dealing with infraspecific name
  output.infraspname<-ifelse(results.output$"dwc:taxonRank"=="forma",
                                    results.output$infraspecificEpithet_forma,
                                    ifelse(results.output$"dwc:taxonRank"=="variety",
                                           results.output$infraspecificEpithet_variety,
                                           ifelse(results.output$"dwc:taxonRank"=="subspecies",
                                                  results.output$infraspecificEpithet_subspecies,"")))
  output.clean.names<-paste(results.output$"dwc:genus",results.output$"dwc:specificEpithet",output.infraspname)
  output.clean.names<-trimws(output.clean.names)

input.clean.name<-paste(genus,species)

output.match.indices<-output.clean.names==input.clean.name

#only retain exact matches if asked.
if(exact.matches.only){
  if(sum(output.match.indices)==0){stop("No exact matches found")
    }else{
            results.output<-results.output[output.match.indices,];
                        output.match.indices=TRUE
    }
}

long.name<-algaebase_output_parse(results.output,"dwc:scientificName")
taxonomic.status<-algaebase_output_parse(results.output,"dwc:taxonomicStatus")
taxonomic.genus<-algaebase_output_parse(results.output,"dwc:genus")
taxonomic.species<-algaebase_output_parse(results.output,"dwc:specificEpithet")
taxonRank<-algaebase_output_parse(results.output,"dwc:taxonRank")
authorship<-algaebase_output_parse(results.output,"dwc:scientificNameAuthorship")
mod.date<-algaebase_output_parse(results.output,"dcterms:modified")
mod.date<-lubridate::ymd(mod.date) #set as date type so you can sort output based on latest modification
currently.accepted=ifelse(taxonomic.status=='currently accepted taxonomically',1,0)
#need to clean up accepted name section.
accepted.name<-ifelse(currently.accepted==1,
                      algaebase_output_parse(results.output,"dwc:scientificName"),
                      algaebase_output_parse(results.output,"dwc:acceptedNameUsage"))

for(i in 1:length(accepted.name)){
  accepted.name[i]<-gsub(authorship[i],"",accepted.name[i],fixed = TRUE)
  accepted.name[i]<-gsub(r"{\s*\([^*].*}","",accepted.name[i]) #remove text after parens
  accepted.name[i]<-trimws(accepted.name[i])
}

input.name=input.clean.name
input.match=ifelse(output.match.indices,1,0)

output<-data.frame(genus=taxonomic.genus,species=taxonomic.species,infrasp=NA,taxonomic.status,currently.accepted,genus.only=0,accepted.name,input.name=input.clean.name,
                   input.match,taxon.rank=taxonRank,mod.date,long.name,authorship)

for(i in 1:nrow(output)){
  output$infrasp[i]<-ifelse(output$taxon.rank[i]=='variety',results.output$infraspecificEpithet_variety[i],
                         ifelse(output$taxon.rank[i]=='forma',results.output$infraspecificEpithet_forma[i],
                                ifelse(output$taxon.rank[i]=='subspecies',results.output$infraspecificEpithet_subspecies[i],NA)))
}



  if(higher){ #merge higher taxonomy and reorder names of output variable
    higher.taxonomy<-algaebase_genus_search(genus,
                                            return.higher.only = TRUE,
                                            handle=handle,exact.matches.only = TRUE,newest.only = TRUE);

    output<-merge(higher.taxonomy,output,all.y=TRUE,by='genus',sort=FALSE);
    output<-subset(output,select= c('accepted.name','input.name','input.match','currently.accepted','genus.only','kingdom','phylum','class','order','family','genus','species','infrasp',
                                    'long.name','taxonomic.status','taxon.rank','mod.date','authorship'))}else{
    output<-subset(output,select=c('accepted.name','input.name','input.match','currently.accepted','genus.only','genus','species','infrasp',
                                                                     'long.name','taxonomic.status','taxon.rank','mod.date','authorship') )

                                    }
  if(newest.only){
    output<-output[output$mod.date==max(output$mod.date),] #only retain the most recent edit
  }else{
    output<-output[order(output$mod.date,decreasing=TRUE),]
  }


  if(!long){output<-output[,names(output) %in% c('long.name','authorship','taxonomic.status','mod.date')==FALSE]}



  return(output)

}

#' Wrapper function for applying genus_search_itis and species_search_itis
#' to a whole data.frame containing scientific names
#' @param df data.frame containing names to check
#' @param namecol integer or character string with column name containing
#' 		  species or genus names
#' @param higher Boolean. If TRUE, add higher taxonomic classifications to output
#' @param genus.only boolean If TRUE, search for matches with just the genus name using genus_search_itis
#'
#' @export itis_search_df
#'
#' @return data.frame with submitted names (orig.name), matched names (matched.name),
#' 1/0 flag indicating that original name is currently accepted (orig.name.accepted),
#' 1/0 flag indicating if search was genus_only (for distinguishing genus_search_itis
#' and species_search_itis results), synonyms if any, and higher taxonomy (if
#' higher=TRUE)
#'
#' @examples
#' \donttest{
#' data(lakegeneva)
#' new.lakegeneva <- genus_species_extract(lakegeneva[1,],'phyto_name')
#' new.lakegeneva$genus_species <- trimws(paste(new.lakegeneva$genus,
#' new.lakegeneva$species))
#' lakegeneva.genus.itischeck <-
#'          itis_search_df(new.lakegeneva,"genus_species")
#' lakegeneva.genus.itischeck
#' }

itis_search_df<-function(df,namecol=NA,higher=FALSE,genus.only=FALSE)
{
	suppressWarnings(rm(list="itis.df")) #remove if present
	itis.df<-data.frame(matched.name=NULL,match=NULL,orig.name.accepted=NULL,
	orig.name=NULL,genus.only=NULL, synonyms=NULL)
		if(higher){
			itis.df<-rbind(itis.df,data.frame(Kingdom=NULL, Subkingdom=NULL, Infrakingdom=NULL, Phylum=NULL, Class=NULL,Subclass=NULL,Order=NULL,
			Family=NULL)
		)
		}

	#wrapper for genspp_itis_search and genus_itis_search on whole data.frame

	nrows<-nrow(df)
	for(i in 1:nrows)
	{
		name=df[[namecol]][i]
		if(!is.na(name) & !is.null(name))
		{
			if(genus.only){
				tmp<-genus_search_itis(name,higher=higher)
			}else{
				tmp<-species_search_itis(name,higher=higher)
			}

		}else{
			tmp<-data.frame(matched.name=NA,match=0,orig.name.accepted=0,
			orig.name=NA,genus.only=ifelse(genus.only,1,0),synonyms="")
			if(higher){
				tmp<-cbind(tmp,data.frame(Kingdom=NA, Subkingdom=NA, Infrakingdom=NA, Phylum=NA, Class=NA,Subclass=NA,Order=NA,
				Family=NA)
				)
			}
		}
		itis.df<-rbind(itis.df,tmp)
	}
	return(itis.df)

}

#' Get value of algaebase API key from a file 
#' @param keyfile path to text file
#'
#' @export get_apikey_fromfile
#' @return api key as character string (invisibly)
#'
#' @examples \dontrun{apikey<-get_apikey_fromfile("keyfile.txt")}

get_apikey_fromfile<-function(keyfile)
{
  #keyfile should be a text file with the API key on the first line
  apikey<-readLines(con=keyfile,n=1,warn=FALSE)
  invisible(apikey) #will invisibly return result
  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/traits_to_mfg.R
\name{traits_to_mfg}
\alias{traits_to_mfg}
\title{Assign MFG based on binary functional traits and taxonomy (Class and Order)}
\usage{
traits_to_mfg(
  flagella = NA,
  size = NA,
  colonial = NA,
  filament = NA,
  centric = NA,
  gelatinous = NA,
  aerotopes = NA,
  class = NA,
  order = NA
)
}
\arguments{
\item{flagella}{1 if flagella are present, 0 if they are absent.}

\item{size}{Character string: 'large' or 'small'. Classification criteria is left to the user.}

\item{colonial}{1 if typically colonial growth form, 0 if typically unicellular.}

\item{filament}{1 if dominant growth form is filamentous, 0 if not.}

\item{centric}{1 if diatom with centric growth form, 0 if not. NA for  non-diatoms.}

\item{gelatinous}{1 mucilagenous sheath is typically present, 0 if not.}

\item{aerotopes}{1 if aerotopes allowing buoyancy regulation are typically present, 0 if not.}

\item{class}{Character string: The taxonomic class of the species}

\item{order}{Character string: The taxonomic order of the species}
}
\value{
A character string of the species' morphofunctional group
}
\description{
Assign MFG based on binary functional traits and taxonomy (Class and Order)
}
\examples{
traits_to_mfg(flagella = 1,size = "large",colonial = 1,filament = 0,centric = NA,gelatinous = 0,
               aerotopes = 0,class = "Euglenophyceae",order = "Euglenales")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accum.R
\name{accum}
\alias{accum}
\title{Split a dataframe column with binomial name into genus and species columns.
Plots change in species richness over time, generates species accumulation curve, and
compares SAC against simulated idealized curve assuming all unique taxa have equal probability
of being sampled at any point in the time series. (author Dietmar Straile)}
\usage{
accum(
  b_data,
  phyto_name = "phyto_name",
  column = NA,
  n = 100,
  save.pdf = FALSE,
  lakename = "",
  datename = "date_dd_mm_yy",
  dateformat = "\%d-\%m-\%y"
)
}
\arguments{
\item{b_data}{Name of data.frame object}

\item{phyto_name}{Character string: field containing phytoplankton id (species, genus, etc.)}

\item{column}{column name or number for field containing abundance (biomass,biovol, etc.).
Can be NA if the dataset only contains a species list for each sampling date.}

\item{n}{number of simulations for randomized ideal species accumulation curve}

\item{save.pdf}{TRUE/FALSE- should plots be displayed or saved to a pdf?}

\item{lakename}{optional character string for adding lake name to pdf output}

\item{datename}{character string name of b_data field containing date}

\item{dateformat}{character string: posix format for datename column}
}
\value{
a two panel plot with trends in richness on top, and cumulative richness vs. simulated
        accumulation curve on bottom
}
\description{
Split a dataframe column with binomial name into genus and species columns.
Plots change in species richness over time, generates species accumulation curve, and
compares SAC against simulated idealized curve assuming all unique taxa have equal probability
of being sampled at any point in the time series. (author Dietmar Straile)
}
\examples{
data(lakegeneva)
#example dataset with 50 rows
head(lakegeneva)

accum(b_data=lakegeneva,column='biovol_um3_ml',n=10,save.pdf=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csrTraits.r
\docType{data}
\name{csrTraits}
\alias{csrTraits}
\title{Database of functional traits for MFG classification, derived from Rimet et al. 2019}
\format{
A data frame with columns:
\describe{
 \item{phyto_name}{binomial scientific name}
 \item{genus}{genus name}
 \item{species}{species name}
 \item{SAV}{surface area:volume ratio}
 \item{MLD}{maximum linear dimension (micrometers)}
 \item{MSV}{product of SAV and MLD; unitless}
\item{volume.um3}{cell or colony biovolume}
\item{surface.area.um2}{biological unit (cell or colony) surface area accounting for mucilage}
\item{Colonial}{1/0 indicates colonial growth form}
\item{Number.of.cells.per.colony}{literature-based average colony abundance}
\item{Geometrical.shape.of.the.colony}{Shape descriptions. See Rimet et al. 2019 for abbreviations}
 \item{traitCSR}{CSR classification using traits_to_CSR function and criteria from Reynolds 2006}
}
}
\usage{
data(mfgTraits)
}
\description{
Database of functional traits for MFG classification, derived from Rimet et al. 2019
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/traits_to_csr.r
\name{traits_to_csr}
\alias{traits_to_csr}
\title{Assign phytoplankton species to CSR functional groups, based on surface to volume ratio and
maximum linear dimension ranges proposed by Reynolds et al. 1988;2006}
\usage{
traits_to_csr(
  sav,
  msv,
  msv.source = "Reynolds 2006",
  traitrange = algaeClassify::traitranges
)
}
\arguments{
\item{sav}{numeric estimate of cell or colony surface area /volume ratio}

\item{msv}{numeric product of surface area/volume ratio and maximum linear dimension}

\item{msv.source}{character string with reference source for distinguishing criteria}

\item{traitrange}{data frame with trait criteria for c,s,r groups. The included table
can be replaced with user-defined criteria if desired. Measurements are:
Surface area/volume ratio (sav), maximum linear dimension (mld) and mld*sav (msv).}
}
\value{
a character string with one of 5 return values: C,CR,S,R, or SR.
        CR and SR groups reflect overlap between criteria for the 3 main groups.
}
\description{
Assign phytoplankton species to CSR functional groups, based on surface to volume ratio and
maximum linear dimension ranges proposed by Reynolds et al. 1988;2006
}
\examples{

traits_to_csr(sav=0.2,msv=10,msv.source='Reynolds 2006',traitrange=traitranges)


}
\seealso{
{<https://powellcenter.usgs.gov/geisha>} for project information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mfg_csr_convert_df.r
\name{mfg_csr_convert_df}
\alias{mfg_csr_convert_df}
\title{Returns a CSR classification based on Morphofunctional group (MFG).
Correspondence based on Salmaso et al. 2015 and Reynolds et al. 1988}
\usage{
mfg_csr_convert_df(phyto.df, mfg)
}
\arguments{
\item{phyto.df}{dataframe containing a character field containing MFG classifications}

\item{mfg}{Character string with MFG name, following Salmaso et al. 2015}
}
\value{
A dataframe with an additional field named CSR, containing CSR classifications or NA
}
\description{
Returns a CSR classification based on Morphofunctional group (MFG).
Correspondence based on Salmaso et al. 2015 and Reynolds et al. 1988
}
\examples{

data(lakegeneva)
lakegeneva<-genus_species_extract(lakegeneva,'phyto_name')
lakegeneva<-species_to_mfg_df(lakegeneva)
lakegeneva<-mfg_csr_convert_df(lakegeneva,mfg='MFG')
head(lakegeneva)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampeff.r
\name{sampeff}
\alias{sampeff}
\title{Visually assess change in sampling effort over time (author: Dietmar Straile)}
\usage{
sampeff(
  b_data,
  column,
  save.pdf = FALSE,
  lakename = "",
  datecolumn = "date_dd_mm_yy",
  dateformat = "\%d-\%m-\%y"
)
}
\arguments{
\item{b_data}{Name of data.frame object}

\item{column}{column name or number for field containing abundance (biomass,biovol, etc.)
can be NA for presence absence}

\item{save.pdf}{TRUE/FALSE Should the output plot be saved to a file? defaults to FALSE}

\item{lakename}{Character string for labeling output plot}

\item{datecolumn}{Character String or number specifying dataframe field with date information}

\item{dateformat}{Character string specifying POSIX data format}
}
\value{
a time-series plot of minimum relative abundance over time. This should change
        systematically with counting effort.
}
\description{
Visually assess change in sampling effort over time (author: Dietmar Straile)
}
\examples{
data(lakegeneva)
#example dataset with 50 rows

sampeff(lakegeneva,column=6) #column 6 contains biovolume
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_apikey_from_file.r
\name{get_apikey_fromfile}
\alias{get_apikey_fromfile}
\title{Get value of algaebase API key from a file}
\usage{
get_apikey_fromfile(keyfile)
}
\arguments{
\item{keyfile}{path to text file}
}
\value{
api key as character string (invisibly)
}
\description{
Get value of algaebase API key from a file
}
\examples{
\dontrun{apikey<-get_apikey_fromfile("keyfile.txt")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mfgTraits.r
\docType{data}
\name{mfgTraits}
\alias{mfgTraits}
\title{Functional Trait Database derived from Rimet et al.}
\format{
A data frame with columns:
\describe{
 \item{phyto_name}{binomial scientific name}
 \item{genus}{genus name}
 \item{species}{species name}
 \item{Mobility.apparatus}{1/0 indicates presence/absence of flagella or motility}
 \item{Size}{character values 'large' or 'small'; based on 35 micrometer max linear dimension}
 \item{Colonial}{1/0 indicates typical colonial growth form or not}
\item{Filament}{1/0 indicates filamentous growth form or not}
\item{Centric}{1/0 indicates diatoms with centric growth form}
\item{Gelatinous}{1/0 indicates presence/absence of mucilage}
\item{Aerotopes}{1/0 indicates presence/absence of aerotopes}
\item{Class}{Taxonomic class}
 \item{Order}{Taxonomic order}
\item{MFG.fromtraits}{MFG classification using traits_to_mfg function}
}
}
\usage{
data(mfgTraits)
}
\description{
Functional Trait Database derived from Rimet et al.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/algaebase_search_df.r
\name{algaebase_search_df}
\alias{algaebase_search_df}
\title{Search algaebase for information about a list of phytoplankton names}
\usage{
algaebase_search_df(
  df,
  apikey = NULL,
  handle = NULL,
  genus.only = FALSE,
  genus.name = "genus",
  species.name = "species",
  higher = TRUE,
  print.full.json = FALSE,
  long = FALSE,
  exact.matches.only = TRUE,
  api_file = NULL,
  sleep.time = 1
)
}
\arguments{
\item{df}{data frame containing columns with genus and species names}

\item{apikey}{valid key for algaebase API as character string}

\item{handle}{curl handle with API key. Will be created if not present.}

\item{genus.only}{boolean: should searches be based solely on the genus name?}

\item{genus.name}{name of data.frame column that contains genus names}

\item{species.name}{name of data.frame column that contains species names}

\item{higher}{boolean should higher taxonomy be included in output?}

\item{print.full.json}{boolean returns raw json output if TRUE. Default is FALSE (return R data frame)}

\item{long}{boolean return long output including full species name and authorship, and entry date from algaebase.}

\item{exact.matches.only}{boolean should results be limited to exact matches?}

\item{api_file}{path to text file containing a valid API key}

\item{sleep.time}{delay between algaebase queries (in seconds). Should be at least 1 second if querying more than 10 names at once.}
}
\value{
data frame that may include: accepted.name (currently accepted synonym if different from input name), input.name (name supplied by user), input.match (1 if exact match, else 0), currently.accepted (1=TRUE/0=FALSE), genus.only (1=genus search/0=genus+species search),higher taxonomy (kingdom,phylum,class,order,family), genus, species (always NA for genus search), infraspecies name (always NA for genus search), long.name (includes author and date if given), taxonomic.status (currently accepted, synonym, or unverified), taxon.rank (taxonomic rank of accepted name (genus, species, infraspecies), mod.date (date when entry was last modified in algaebase).
}
\description{
Search algaebase for information about a list of phytoplankton names
}
\examples{
\dontrun{
data(lakegeneva)
#example dataset with 50 rows

new.lakegeneva <- genus_species_extract(lakegeneva,'phyto_name')
lakegeneva.algaebase<-algaebase_search_df(new.lakegeneva[1:10,],higher=TRUE,long=TRUE)
head(lakegeneva.algaebase)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/traitranges.r
\docType{data}
\name{traitranges}
\alias{traitranges}
\title{surface/volume ratio and max linear dimension criteria for CSR
From Reynolds 1988 and Reynolds 2006}
\format{
A data frame with columns:
\describe{
 \item{Measurement}{measurement type}
 \item{C.min}{minimum value for C}
 \item{S.min}{minimum value for S}
 \item{R.min}{minimum value for R}
 \item{C.max}{maximum value for C}
 \item{S.max}{maximum value for S}
 \item{R.max}{maximum value for R}
 \item{units}{units of measurement}
 \item{source}{source for criteria}
}
}
\usage{
data(traitranges)
}
\description{
surface/volume ratio and max linear dimension criteria for CSR
From Reynolds 1988 and Reynolds 2006
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/species_mfg_library.r
\docType{data}
\name{species_mfg_library}
\alias{species_mfg_library}
\title{Trait-based MFG classifications for common Eurasion/North American phytoplankton species.
See accompanying manuscript for sources}
\format{
A data frame with columns:
\describe{
 \item{genus}{genus name}
 \item{species}{species name}
 \item{MFG}{corresponding MFG classification based on Salmaso et al. 2015}
 \item{source}{literature or online source for MFG classification}
 }
}
\usage{
data(species_mfg_library)
}
\description{
Trait-based MFG classifications for common Eurasion/North American phytoplankton species.
See accompanying manuscript for sources
}
\references{
{Algaebase} \url{https://www.algaebase.org}

{Phycokey} \url{https://www.cfb.unh.edu/phycokey/phycokey.htm}

{Western Diatoms of North America} \url{https://diatoms.org}

{CyanoDB 2} \url{http://www.cyanodb.cz/}

{Nordic Microalgae} \url{https://nordicmicroalgae.org}

{Phytopedia} \url{https://phytoplankton.eoas.ubc.ca/}

{Kapustin, D., Sterlyagova, I. and Patova, E., 2019. Morphology of Chrysastrella paradoxa stomatocysts from the Subpolar Urals (Russia) with comments on related morphotypes. Phytotaxa, 402(6), pp.295-300.}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bestmatch.r
\name{bestmatch}
\alias{bestmatch}
\title{fuzzy partial matching between a scientific name and a list of possible matches}
\usage{
bestmatch(enteredName, possibleNames, maxErr = 3, trunc = TRUE)
}
\arguments{
\item{enteredName}{Character string with name to check}

\item{possibleNames}{Character vector of possible matches}

\item{maxErr}{maximum number of different bits allowed for a partial match}

\item{trunc}{TRUE/FALSE. if true and no match, retry with last three letters truncated}
}
\value{
a character string with the best match, or 'multiplePartialMatches'
}
\description{
fuzzy partial matching between a scientific name and a list of possible matches
}
\examples{
possibleMatches=c('Viburnum edule','Viburnum acerifolia')
bestmatch(enteredName='Viburnum edulus',possibleNames=possibleMatches)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lakegeneva.R
\docType{data}
\name{lakegeneva}
\alias{lakegeneva}
\title{example dataset from lake Geneva, Switzerland}
\format{
A data frame with columns:
\describe{
 \item{lake}{lake name}
 \item{phyto_name}{phytoplankton species name}
 \item{month}{month of sampling}
 \item{year}{year of sampling}
 \item{date_dd_mm_yy}{date of sampling}
 \item{biovol_um3_ml}{biovolume}
}
}
\usage{
data(lakegeneva)
}
\description{
example dataset from lake Geneva, Switzerland
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/traits_to_csr_df.r
\name{traits_to_csr_df}
\alias{traits_to_csr_df}
\title{Add CSR functional group classifications to a dataframe of phytoplankton species, based on surface to volume ratio and
maximum linear dimension ranges proposed by Reynolds et al. 1988;2006}
\usage{
traits_to_csr_df(
  df,
  sav,
  msv,
  msv.source = "Reynolds 2006",
  traitrange = algaeClassify::traitranges
)
}
\arguments{
\item{df}{name of dataframe}

\item{sav}{character string with name of column that contains surface to volume ratio values}

\item{msv}{character string with name of column that contains maximum linear dimension * surface to volume ratio values}

\item{msv.source}{character string with reference source for distinguishing criteria}

\item{traitrange}{data frame with trait criteria for c,s,r groups. The included table
can be replaced with user-defined criteria if desired. Measurements are:
Surface area/volume ratio (sav), maximum linear dimension (mld) and mld*sav (msv).}
}
\value{
a character string with one of 5 return values: C,CR,S,SR, or R
}
\description{
Add CSR functional group classifications to a dataframe of phytoplankton species, based on surface to volume ratio and
maximum linear dimension ranges proposed by Reynolds et al. 1988;2006
}
\examples{

csr.df<-data.frame(msv=10,sav=1)

csr.df$CSR<-traits_to_csr_df(csr.df,'msv','sav')

print(csr.df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_apikey_from_Renviron.r
\name{get_apikey}
\alias{get_apikey}
\title{Get value of algaebase API key from Environment variable
Return an error if variable not set.}
\usage{
get_apikey()
}
\value{
api key as character string (invisibly)
}
\description{
Get value of algaebase API key from Environment variable
Return an error if variable not set.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_mat.R
\name{mean_naomit}
\alias{mean_naomit}
\title{Compute mean value while ignoring NA's}
\usage{
mean_naomit(x)
}
\arguments{
\item{x}{A numeric vector that may contain NA's}
}
\value{
the mean value
}
\description{
Compute mean value while ignoring NA's
}
\examples{
data(lakegeneva)
#example dataset with 50 rows

mean_naomit(lakegeneva$biovol_um3_ml)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_mat.R
\name{date_mat}
\alias{date_mat}
\title{Transform a phytoplankton timeseries into a matrix of abundances for ordination}
\usage{
date_mat(
  phyto.df,
  abundance.var = "biovol_um3_ml",
  summary.type = "abundance",
  taxa.name = "phyto_name",
  date.name = "date_dd_mm_yy",
  format = "\%d-\%m-\%y",
  time.agg = c("day", "month", "year", "monthyear"),
  fun = mean_naomit
)
}
\arguments{
\item{phyto.df}{Name of data.frame object}

\item{abundance.var}{Character string: field containing abundance data.
Can be NA if the dataset only contains a species list for each sampling date.}

\item{summary.type}{'abundance' for a matrix of aggregated abundance,'presence.absence'
for 1 (present) and 0 (absent).}

\item{taxa.name}{Character string: field containing taxonomic identifiers.}

\item{date.name}{Character string: field containing date.}

\item{format}{Character string: POSIX format string for formatting date column.}

\item{time.agg}{Character string: time interval for aggregating abundance. default is day.}

\item{fun}{function for aggregation. default is mean, excluding NA's.}
}
\value{
A matrix of phytoplankton abundance, with taxa in rows and time in columns.
        If time.agg = 'monthyear', returns a 3dimensional matrix (taxa,month,year).
        If abundance.var = NA, matrix cells will be 1 for present, 0 for absent
}
\description{
Transform a phytoplankton timeseries into a matrix of abundances for ordination
}
\examples{
data(lakegeneva)
#example dataset with 50 rows

geneva.mat1<-date_mat(lakegeneva,time.agg='month',summary.type='presence.absence')
geneva.mat2<-date_mat(lakegeneva,time.agg='month',summary.type='abundance')

geneva.mat1
geneva.mat2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/traits_to_mfg_df.r
\name{traits_to_mfg_df}
\alias{traits_to_mfg_df}
\title{Assign morphofunctional groups to a dataframe of functional traits and higher taxonomy}
\usage{
traits_to_mfg_df(
  dframe,
  arg.names = c("flagella", "size", "colonial", "filament", "centric", "gelatinous",
    "aerotopes", "class", "order")
)
}
\arguments{
\item{dframe}{An R dataframe containing functional trait information and higher taxonomy}

\item{arg.names}{Character string of column names corresponding to arguments for traits_to_mfg()}
}
\value{
A character vector containing morpho-functional group (MFG) designations
}
\description{
Assign morphofunctional groups to a dataframe of functional traits and higher taxonomy
}
\examples{
#create a two-row example dataframe of functional traits
func.dframe=data.frame(flagella=1,size=c("large","small"),colonial=0,filament=0,centric=NA,
                       gelatinous=0,aerotopes=0,class="Euglenophyceae",order="Euglenales",
                       stringsAsFactors=FALSE)

#check the dataframe
print(func.dframe)

#run the function to produce a two-element character vector
func.dframe$MFG<-traits_to_mfg_df(func.dframe,c("flagella","size","colonial",
                                 "filament","centric","gelatinous",
                                 "aerotopes","class","order"))

print(func.dframe)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genus_species_extract.r
\name{genus_species_extract}
\alias{genus_species_extract}
\title{Split a dataframe column with binomial name into genus and species columns.}
\usage{
genus_species_extract(phyto.df, phyto.name)
}
\arguments{
\item{phyto.df}{Name of data.frame object}

\item{phyto.name}{Character string: field in phyto.df containing species name.}
}
\value{
A data.frame with new character fields 'genus' and 'species'
}
\description{
Split a dataframe column with binomial name into genus and species columns.
}
\examples{
data(lakegeneva)
#example dataset with 50 rows

head(lakegeneva) #need to split the phyto_name column
new.lakegeneva=genus_species_extract(lakegeneva,'phyto_name')

head(new.lakegeneva)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gnr_df.r
\name{gnr_df}
\alias{gnr_df}
\title{Wrapper function to apply gnr_simple across a data.frame or list of species
names. Modified on 11/18/2025 by Vijay Patil (vpatil@usgs.gov) for
algaeClassify v2.0.5 (pending approval on CRAN).}
\usage{
gnr_df(
  df,
  name.column,
  sourceid = NULL,
  fuzzy_uninomial = TRUE,
  name_type = "canonical_full",
  higher = FALSE
)
}
\arguments{
\item{df}{data.frame containing names to check}

\item{name.column}{integer or character string with column name containing
species names}

\item{sourceid}{integer vector with data source ids.
see https://resolver.globalnames.org/sources/}

\item{fuzzy_uninomial}{boolean. Use fuzzy matching for uninomial names?}

\item{name_type}{Specify format of matched names. Options are
'canonical_simple' (canonical binomial name), 'canonical_full' (with subspecies
or subgenera), or 'with_context' (with author and year appended).}

\item{higher}{boolean: Return higher taxonomic classifications?}
}
\value{
new data.frame original names (input_name), information about match
type,the best match (match_name), taxonomic status, and other output from
gnr_simple(). Will contain a row of NAs if no matches were found for a name.
}
\description{
Provides convienent output with a row per name, to streamline merging with
original data.
}
\examples{
data(lakegeneva)
#example dataset with 50 rows

lakegeneva<- genus_species_extract(lakegeneva,'phyto_name')
lakegeneva$genus_species <- trimws(paste(lakegeneva$genus,
lakegeneva$species))

#checking for matches from all GNRS sources, first 5 rows:
lakegeneva.namematches <- gnr_df(lakegeneva,"genus_species")
lakegeneva.namematches
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genus_search_itis.r
\name{genus_search_itis}
\alias{genus_search_itis}
\title{Wrapper function for several functions in ritis::
Searches ITIS database for matches to a genus name}
\usage{
genus_search_itis(genus, higher = FALSE)
}
\arguments{
\item{genus}{Character string. genus name to search for in ITIS}

\item{higher}{Boolean. If TRUE, add higher taxonomic classifications to output}
}
\value{
input data.frame with matches, current accepted names, synonyms, and higher taxonomy
}
\description{
Wrapper function for several functions in ritis::
Searches ITIS database for matches to a genus name
}
\examples{
\donttest{
genus='Anabaena'
genus_search_itis(genus,higher=FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phyto_ts_aggregate.r
\name{phyto_ts_aggregate}
\alias{phyto_ts_aggregate}
\title{Aggregate phytoplankton timeseries based on abundance. Up to 3 grouping variables can be given:
e.g. genus, species, stationid, depth range.
If no abundance var is given, will aggregate to presence/absence of grouping vars.}
\usage{
phyto_ts_aggregate(
  phyto.data,
  DateVar = "date_dd_mm_yy",
  SummaryType = c("abundance", "presence.absence"),
  AbundanceVar = "biovol_um3_ml",
  GroupingVar1 = "phyto_name",
  GroupingVar2 = NA,
  GroupingVar3 = NA,
  remove.rare = FALSE,
  fun = sum,
  format = "\%d-\%m-\%y"
)
}
\arguments{
\item{phyto.data}{data.frame}

\item{DateVar}{character string: field name for date variable. character or POSIX data.}

\item{SummaryType}{'abundance' for a matrix of aggregated abundance,'presence.absence'
for 1 (present) and 0 (absent).}

\item{AbundanceVar}{character string with field name containing abundance data
Can be NA if data is only a species list and aggregated presence/absence is desired.}

\item{GroupingVar1}{character string: field name for first grouping variable. defaults to spp.}

\item{GroupingVar2}{character string: name of additional grouping var field}

\item{GroupingVar3}{character string: name of additional grouping var field}

\item{remove.rare}{TRUE/FALSE. If TRUE, removes all instances of GroupingVar1 that occur < 5%
of time periods.}

\item{fun}{function used to aggregate abundance based on grouping variables}

\item{format}{character string: format for DateVar POSIXct conversion}
}
\value{
a data.frame with grouping vars, date_dd_mm_yy, and abundance or presence/absence
}
\description{
Aggregate phytoplankton timeseries based on abundance. Up to 3 grouping variables can be given:
e.g. genus, species, stationid, depth range.
If no abundance var is given, will aggregate to presence/absence of grouping vars.
}
\examples{
data(lakegeneva)
lakegeneva<-genus_species_extract(lakegeneva,'phyto_name')
lg.genera=phyto_ts_aggregate(lakegeneva,SummaryType='presence.absence',
                             GroupingVar1='genus')
head(lg.genera)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mfg_csr.r
\name{mfg_csr_convert}
\alias{mfg_csr_convert}
\title{Returns a CSR classification based on Morphofunctional group (MFG).
Correspondence based on Salmaso et al. 2015 and Reynolds et al. 1988}
\usage{
mfg_csr_convert(mfg)
}
\arguments{
\item{mfg}{Character string with MFG name, following Salmaso et al. 2015}
}
\value{
A character string with values 'C','S','R','CR','SC','SR', or NA
}
\description{
Returns a CSR classification based on Morphofunctional group (MFG).
Correspondence based on Salmaso et al. 2015 and Reynolds et al. 1988
}
\examples{

mfg_csr_convert("11a-NakeChlor")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/algaebase_output_parse.r
\name{algaebase_output_parse}
\alias{algaebase_output_parse}
\title{Helper function for parsing output from algaebase}
\usage{
algaebase_output_parse(x, field.name)
}
\arguments{
\item{x}{list object containing output from an algaebase query}

\item{field.name}{character string}
}
\value{
selected output variable as character vector
}
\description{
Helper function for parsing output from algaebase
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/algaebase_species_search.r
\name{algaebase_species_search}
\alias{algaebase_species_search}
\title{Retrieve taxonomic information from the algaebase online database (www.algaebase.org) based on a user-specified genus and species name . This function requires a valid API key for algaebase.}
\usage{
algaebase_species_search(
  genus,
  species,
  apikey = NULL,
  handle = NULL,
  higher = TRUE,
  print.full.json = FALSE,
  newest.only = TRUE,
  long = FALSE,
  exact.matches.only = TRUE,
  api_file = NULL
)
}
\arguments{
\item{genus}{genus name as character string}

\item{species}{species name as character string}

\item{apikey}{valid key for algaebase API as character string}

\item{handle}{curl handle with API key. Will be created if not present.}

\item{higher}{boolean should higher taxonomy be included in output?}

\item{print.full.json}{boolean returns raw json output if TRUE. Default is FALSE (return R data frame)}

\item{newest.only}{boolean should results be limited to the most recent matching entry in algaebase?}

\item{long}{boolean return long output including full species name and authorship, and entry date from algaebase.}

\item{exact.matches.only}{boolean should results be limited to exact matches?}

\item{api_file}{path to text file containing a valid API key}
}
\value{
data frame that may include: accepted.name (currently accepted synonym if different from input name), input.name (name supplied by user), input.match (1 if exact match, else 0), currently.accepted (1=TRUE/0=FALSE), genus.only (1=genus search/0=genus+species search),higher taxonomy (kingdom,phylum,class,order,family), genus, species (always NA for genus search), infraspecies name (always NA for genus search), long.name (includes author and date if given), taxonomic.status (currently accepted, synonym, or unverified), taxon.rank (taxonomic rank of accepted name (genus, species, infraspecies), mod.date (date when entry was last modified in algaebase).
}
\description{
Retrieve taxonomic information from the algaebase online database (www.algaebase.org) based on a user-specified genus and species name . This function requires a valid API key for algaebase.
}
\examples{

\dontrun{algaebase_species_search("Anabaena flos-aquae")} #not run


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/species_to_mfg.r
\name{species_to_mfg}
\alias{species_to_mfg}
\title{Conversion of a single genus and species name to a single MFG. Uses species.mfg.library}
\usage{
species_to_mfg(genus, species = "", flag = 1, mfgDbase = NA)
}
\arguments{
\item{genus}{Character string: genus name}

\item{species}{Character string: species name}

\item{flag}{Resolve ambiguous mfg: 1 = return(NA),2= manual selection}

\item{mfgDbase}{data.frame of species MFG classifications. Defaults to the supplied species.mfg.library data object}
}
\value{
a data frame with MFG classification and diagnostic information.
ambiguous.mfg=1 if multiple possible mfg matches
genus.classification=1 if no exact match was found with genus + species name
partial.match=1 if mfg was based on fuzzy matching of taxonomic name.
}
\description{
Conversion of a single genus and species name to a single MFG. Uses species.mfg.library
}
\examples{
species_to_mfg('Scenedesmus','bijuga')
#returns "11a-NakeChlor"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mfg_csr_library.r
\docType{data}
\name{mfg_csr_library}
\alias{mfg_csr_library}
\title{MFG-CSR correspondence based on CSR-trait relationships in Reynolds et al. 1988
and MFG-trait relationships in Salmaso et al. 2015}
\format{
A data frame with columns:
\describe{
 \item{MFG}{full MFG name from Salmaso et al. 2015}
 \item{CSR}{CSR classification including intermediate classes}
}
}
\usage{
data(mfg_csr_library)
}
\description{
MFG-CSR correspondence based on CSR-trait relationships in Reynolds et al. 1988
and MFG-trait relationships in Salmaso et al. 2015
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/species_to_mfg_df.r
\name{species_to_mfg_df}
\alias{species_to_mfg_df}
\title{Wrapper function to apply species_phyto_convert() across a data.frame}
\usage{
species_to_mfg_df(phyto.df, flag = 1, mfgDbase = NA)
}
\arguments{
\item{phyto.df}{Name of data.frame. Must have character fields named 'genus' and 'species'}

\item{flag}{Resolve ambiguous MFG: 1 = return(NA), 2 = manual selection}

\item{mfgDbase}{specify library of species to MFG associations.}
}
\value{
input data.frame with a new character column of MFG classifications
and diagnostic information
}
\description{
Wrapper function to apply species_phyto_convert() across a data.frame
}
\examples{
data(lakegeneva)
#example dataset with 50 rows

new.lakegeneva <- genus_species_extract(lakegeneva,'phyto_name')
new.lakegeneva <- species_to_mfg_df(new.lakegeneva)
head(new.lakegeneva)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gnr_simple.r
\name{gnr_simple}
\alias{gnr_simple}
\title{checks species names against a variety of online databases
supports fuzzy partial matching, using the Global Names Resolver
(https://resolver.globalnames.org/). Modified on 11/18/2025 by
Vijay Patil (vpatil@usgs.gov) for
algaeClassify v2.0.5 (pending approval on CRAN).}
\usage{
gnr_simple(
  name,
  sourceid = NULL,
  best_match = TRUE,
  fuzzy_uninomial = TRUE,
  name_type = "canonical_full",
  higher = FALSE
)
}
\arguments{
\item{name}{character string binomial scientific name to resolve}

\item{sourceid}{integer vector with data source ids.
see https://resolver.globalnames.org/sources/}

\item{best_match}{boolean. Should the best match be returned based on score?}

\item{fuzzy_uninomial}{boolean. Use fuzzy matching for uninomial names?}

\item{name_type}{Specify format of matched names. Options are
'canonical_simple' (canonical binomial name), 'canonical_full' (with subspecies
or subgenera), or 'with_context' (with author and year appended).}

\item{higher}{boolean: Return higher taxonomic classifications?}
}
\value{
new data.frame with name matches, column indicating match type
and scores from Global Names Resolver (https://resolver.globalnames.org/).
Will contain a row of NAs if no matches found
}
\description{
checks species names against a variety of online databases
supports fuzzy partial matching, using the Global Names Resolver
(https://resolver.globalnames.org/). Modified on 11/18/2025 by
Vijay Patil (vpatil@usgs.gov) for
algaeClassify v2.0.5 (pending approval on CRAN).
}
\examples{
#Visit https://resolver.globalnames.org/data_sources to see all possible
#data sources for name checking.
name<-"Aphanazomenon flos-aquae"
#sourceid=3 for ITIS database
gnr_simple(name,sourceid=3) #search for best match from ITIS
gnr_simple(name,sourceid=NULL,best_match=FALSE) #search for all matches from any source
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/species_search_itis.r
\name{species_search_itis}
\alias{species_search_itis}
\title{Wrapper function for several functions in ritis::
Searches ITIS database for matches to a binomial scientific name
outputs matches, current accepted names, synonyms, and higher taxonomy}
\usage{
species_search_itis(genspp, higher = FALSE)
}
\arguments{
\item{genspp}{Character string. Binomial scientific name with space between genus
and species.}

\item{higher}{Boolean. If TRUE, add higher taxonomic classifications to output}
}
\value{
data.frame with submitted name (orig.name), matched name (matched.name),
1/0 flag indicating that original name is currently accepted (orig.name.accepted),
1/0 flag indicating if search was genus_only (for distinguishing genus_search_itis
and species_search_itis results), synonyms if any, and higher taxonomy (if
higher=TRUE)
}
\description{
Wrapper function for several functions in ritis::
Searches ITIS database for matches to a binomial scientific name
outputs matches, current accepted names, synonyms, and higher taxonomy
}
\examples{
\donttest{
species="Aphanizomenon flosaquae"
species_search_itis(species,higher=FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/itis_search_df.r
\name{itis_search_df}
\alias{itis_search_df}
\title{Wrapper function for applying genus_search_itis and species_search_itis
to a whole data.frame containing scientific names}
\usage{
itis_search_df(df, namecol = NA, higher = FALSE, genus.only = FALSE)
}
\arguments{
\item{df}{data.frame containing names to check}

\item{namecol}{integer or character string with column name containing
species or genus names}

\item{higher}{Boolean. If TRUE, add higher taxonomic classifications to output}

\item{genus.only}{boolean If TRUE, search for matches with just the genus name using genus_search_itis}
}
\value{
data.frame with submitted names (orig.name), matched names (matched.name),
1/0 flag indicating that original name is currently accepted (orig.name.accepted),
1/0 flag indicating if search was genus_only (for distinguishing genus_search_itis
and species_search_itis results), synonyms if any, and higher taxonomy (if
higher=TRUE)
}
\description{
Wrapper function for applying genus_search_itis and species_search_itis
to a whole data.frame containing scientific names
}
\examples{
\donttest{
data(lakegeneva)
new.lakegeneva <- genus_species_extract(lakegeneva[1,],'phyto_name')
new.lakegeneva$genus_species <- trimws(paste(new.lakegeneva$genus,
new.lakegeneva$species))
lakegeneva.genus.itischeck <-
         itis_search_df(new.lakegeneva,"genus_species")
lakegeneva.genus.itischeck
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_algaebase_apikey_header.r
\name{set_algaebase_apikey_header}
\alias{set_algaebase_apikey_header}
\title{Add algaebase API key to curl handle}
\usage{
set_algaebase_apikey_header(apikey = NULL)
}
\arguments{
\item{apikey}{character string with valid key}
}
\value{
curl handle object
}
\description{
Add algaebase API key to curl handle
}
